/*
 * yaepg.c: A plugin for the Video Disk Recorder
 *
 * See the README file for copyright information and how to reach the author.
 *
 * $Id: yaepg.c,v 1.15 2004/11/08 01:41:20 bball Exp $
 */

#include <vdr/device.h>
#ifdef FAVORITES_AND_SORT_PATCH
#include <vdr/favorites.h>
#endif
#include <vdr/osd.h>
#include <vdr/themes.h>
#include <ctype.h>
#include "yaepg.h"
#include "yaepgSetupMenu.h"

int iWhite;
int iGold;
int iCyan;
int iBlue;
int iUpperBackdrop;
int iLowerBackdrop;
int iIdent;
int iHighlight;
int iHighlighter;
int iTextHighlighter;
int iTint;

int iOutputResW;  
int iOutputResH;  
int iChannelCount;

int iTop;
int iLeftMarginLimit;
int iEPGMarginX;
int iChanWidthModifer;
int iTimeWidthModifer;
int iEventWidthModifer;
int iEPGMarginY;
int iEPGMainMarginY;
int iInfboxOffset;
int iSrcboxWidth;
int iTxtboxWidth;
int iInfboxTxtWidth;
int iInfboxWidth;
int iIdboxWidth;
int iDateboxWidth;
#ifdef XINE_UI_ZOOM_PATCH
int iZoomToPrevious;
int iDeinterlaceState;
#endif

int iShowAll;
#ifdef FAVORITES_AND_SORT_PATCH
int iHasFavorites;
int iShowFavorites;
int iSrcHasFavorite;
#endif
int iChanSource;
int iLastSource;
int source_list[TOTALSOURCES];

unsigned int uiChannelEntryTimeout;

#ifdef XINE_UI_ZOOM_PATCH
char cPreviousZoom[8];
char cPreviousDeinterlace[8];
#endif


class cTextBox {
private:
	int		width;
	int		height;
	int		xBorder;
	int		yBorder;
	int		xCoord;
	int		yCoord;

	tColor		fgColor;
	tColor		txColor;
	tColor		bgColor;
	tColor		hlColor;
	bool		hl;
	bool		tx;

	const cFont	*font;
	eDvbFont	fontC;
	int		xAlign;
	int		ySpacing;

	int		numLines;
	int		startNum;
	char		*text[TBOX_MAX_LINES];
	char		*abrevtext;

	cBitmap	*bitmap;
	cBitmap	*destBitmap;

protected:
	bool		initialized;
	int		arrow;

public:
	cTextBox(void);
	cTextBox(int, int, tColor, tColor, eDvbFont);
	~cTextBox();
	void	Init(int, int, tColor, tColor, eDvbFont);
	int	Height(void);
	int	Width(void);
	int	SetText(int, ...);
	void	AddLine(char *);
	int	SetXAlign(int);
	int	Draw(int, int, cBitmap *);
	int	Draw(void);
	int	SetDest(int, int, cBitmap *);
	int	SetHLColor(tColor, tColor);
	int	SetHighLight(bool, bool);
	void	SetYSpacing(int yspc) { ySpacing = yspc; }
	void	DrawLine(int, int, int, int, tColor);
};

cTextBox::cTextBox(void)
{
}

cTextBox::cTextBox(int w, int h, tColor fg, tColor bg, eDvbFont f)
{
	width = w;
	height = h;
	xBorder = TBOX_DEF_XBORDER;
	yBorder = TBOX_DEF_YBORDER;
	xCoord = 0;
	yCoord = 0;

	fgColor = fg;
	txColor = fg;
	bgColor = bg;
	hlColor = bg;
	hl = false;
	tx = false;
	font = cFont::GetFont(f);
	fontC = f;
	xAlign = TBOX_DEF_XALIGN;
	ySpacing = TBOX_DEF_YSPACING;
	
	numLines = 0;
	startNum = 0;
	bzero(text, (sizeof(char *) * TBOX_MAX_LINES));
	abrevtext = NULL;

	bitmap = new cBitmap(width, height, 2);
	destBitmap = NULL;

	initialized = true;
	arrow = 0;
}

cTextBox::~cTextBox()
{
	int i = 0;

	for (i = 0; i < numLines; i++) {
		if (text[i] != NULL) {
			free(text[i]);
		}
	}

	if (bitmap) {
		delete bitmap;
	}
}

void
cTextBox::Init(int w, int h, tColor fg, tColor bg, eDvbFont f)
{
	width = w;
	height = h;
	xBorder = TBOX_DEF_XBORDER;
	yBorder = TBOX_DEF_YBORDER;
	xCoord = 0;
	yCoord = 0;

	fgColor = fg;
	txColor = fg;
	bgColor = bg;
	hlColor = bg;
	hl = false;
	tx = false;

	font = cFont::GetFont(f);
	fontC = f;
	xAlign = TBOX_DEF_XALIGN;
	ySpacing = TBOX_DEF_YSPACING;
	
	numLines = 0;
	bzero(text, (sizeof(char *) * TBOX_MAX_LINES));
	abrevtext = NULL;

	bitmap = new cBitmap(width, height, 2);
	destBitmap = NULL;

	initialized = true;
	arrow = 0;
}

int
cTextBox::SetText(int num, ...)
{
	va_list ap;
	int i;

	if (num > TBOX_MAX_LINES) {
		return (-1);
	}

	/*
	 * Free the old text.
	 */
	for (i = 0; i < numLines; i++) {
		if (text[i] != NULL) {
			free(text[i]);
		}
	}

	/*
	 * Copy the new text.
	 */
	va_start(ap, num);
	for (numLines = 0; numLines < num; numLines++) {
		const char *s = va_arg(ap, char *);
		if (s == NULL) {
			s = "";
		}
		text[numLines] = strdup(s);
		if (text[numLines] == NULL) {
			va_end(ap);
			fprintf(stderr, "Out of memory!\n");
			return (-1);
		}
	}
	va_end(ap);

	return (0);
}

void
cTextBox::AddLine(char *s)
{
	if (numLines == TBOX_MAX_LINES) {
		return;
	}

	text[numLines] = strdup(s);
	numLines++;
	return;
}

int
cTextBox::Draw(int bx, int by, cBitmap *destBmp)
{
	int i, x, y, textHeight, textWidth;

	/*
	 * First time this textbox has been draw.  Stash away the bitmap its
	 * being drawn on and its coordinates so it can be redraw later without
	 * having to know this information.
	 */
	if (destBitmap == NULL) {
		xCoord = bx;
		yCoord = by;
		destBitmap = destBmp;
	}

	bitmap->DrawRectangle(0, 0, width, height, (hl ? hlColor : bgColor));

	textHeight = (TBOX_TXT_HEIGHT * numLines) + ((numLines - 1) * ySpacing);
	y = (height - textHeight) / 2;
	if (y < yBorder) {
		y = yBorder;
	}

	for (i = 0; i < numLines; i++) {
		switch (xAlign) {
		case TBOX_TXT_CENTER:
			textWidth = font->Width(text[i]);
			x = (width - textWidth) / 2;
			if (x < xBorder) {
				x = xBorder;
			}
			break;

		case TBOX_TXT_LEFT:
			x = xBorder;
			break;

		case TBOX_TXT_RIGHT:
			textWidth = font->Width(text[i]);
			x = (width - xBorder) - textWidth;
			if (x < xBorder) {
				x = xBorder;
			}
			break;
		default:
			fprintf(stderr, "Unknown text alignment!\n");
			return (-1);
		}

		bitmap->DrawText(x, y, text[i], (tx ? txColor : fgColor), (hl ? hlColor : bgColor), font);
		y += TBOX_TXT_HEIGHT + ySpacing;
	}

	/*
	 * Fill in the right hand border of the box and the bottom border.
	 */
	bitmap->DrawRectangle((width - xBorder), 0, width, height, (hl ? hlColor : bgColor));
	bitmap->DrawRectangle(0, (height - yBorder), width, height, (hl ? hlColor : bgColor));

	/*
	 * Add the arrows.
	 */
	if (arrow & ARROW_LEFT) {
		DrawLine(0, height/2, 8, 0, iWhite);
		DrawLine(0, height/2, 8, height, iWhite);
		DrawLine(1, height/2, 9, 0, iWhite);
		DrawLine(1, height/2, 9, height, iWhite);
	}
	if (arrow & ARROW_RIGHT) {
		DrawLine(width-1, height/2, width-9, 0, iWhite);
		DrawLine(width-1, height/2, width-9, height, iWhite);
		DrawLine(width-2, height/2, width-10, 0, iWhite);
		DrawLine(width-2, height/2, width-10, height, iWhite);
	}

	destBmp->DrawBitmap(bx, by, *bitmap);

	return (0);
}

int
cTextBox::Draw(void)
{
	if (destBitmap != NULL) {
		return Draw(xCoord, yCoord, destBitmap);
	}

	fprintf(stderr, "Trying to draw a textbox that doesn't have dest!\n");
	return -1;
}

int
cTextBox::SetXAlign(int xa)
{
	xAlign = xa;

	return (0);
}

int
cTextBox::Width(void)
{
	return (width);
}

int
cTextBox::Height(void)
{
	return (height);
}

int
cTextBox::SetHLColor(tColor tx, tColor hl)
{
	txColor = tx;
	hlColor = hl;
	return (0);
}

int
cTextBox::SetHighLight(bool t, bool h)
{
	tx = t;
	hl = h;
	return (0);
}

/*
 * This line drawing code was lifted from someone's webpage :-)
 */
void
cTextBox::DrawLine(int x0, int y0, int x1, int y1, tColor color)
{
	int i;
	int steep = 1;
	int sx, sy;
	int dx, dy;
	int e;

	int tmpswap;
#define SWAP(a,b) tmpswap = a; a = b; b = tmpswap;

	dx = abs(x1 - x0);
	sx = ((x1 - x0) > 0) ? 1 : -1;
	dy = abs(y1 - y0);
	sy = ((y1 - y0) > 0) ? 1 : -1;
	if (dy > dx) {
		steep = 0;
		SWAP(x0, y0);
		SWAP(dx, dy);
		SWAP(sx, sy);
	}
	e = (dy << 1) - dx;
	for (i = 0; i < dx; i++) {
		if (steep) {
			bitmap->DrawPixel(y0, x0, color);
		}
		else {
			bitmap->DrawPixel(y0, x0, color);
		}
		while (e >= 0) {
			y0 += sy;
			e -= (dx << 1);
		}
		x0 += sx;
		e += (dy << 1);
	}
}

#define CHBOX_WIDTH				68
#define CHBOX_HEIGHT				36

class cChanBox : public cTextBox {
private:
	cChannel chan;

public:
	cChanBox(void);
	cChanBox(cChannel *);
	~cChanBox();
	int SetChan(cChannel *);
	tChannelID GetChanID(void);
	int GetChanNum(void);
};

cChanBox::cChanBox(void)
	: cTextBox(CHBOX_WIDTH + iChanWidthModifer, CHBOX_HEIGHT + iEPGMarginY, iWhite, iHighlight, fontChanBox)
{
}

cChanBox::cChanBox(cChannel *c)
	: cTextBox(CHBOX_WIDTH + iChanWidthModifer, CHBOX_HEIGHT + iEPGMarginY, iWhite, iHighlight, fontChanBox)
{
	char numStr[8];

	chan = *c;
	SetXAlign(TBOX_TXT_CENTER);
	SetYSpacing(-3);
	snprintf(numStr, 8, "%d", c->Number());
	SetText(2, c->Name(), numStr);
}

cChanBox::~cChanBox()
{
}

int
cChanBox::SetChan(cChannel *c)
{
	char numStr[8];

	chan = *c;
	SetXAlign(TBOX_TXT_CENTER);
	snprintf(numStr, 8, "%d", c->Number());
	SetText(2, c->Name(), numStr);

	return (0);
}

tChannelID
cChanBox::GetChanID(void)
{
	return (chan.GetChannelID());
}

int
cChanBox::GetChanNum(void)
{
	return (chan.Number());
}

#define TMBOX_WIDTH			144
#define TMBOX_HEIGHT			18

class cTimeBox : public cTextBox {
private:
	int hour;
	int minute;

public:
	struct tm tmTime;

	cTimeBox(void);
	cTimeBox(struct tm *, int);
	~cTimeBox();
	int SetTime(struct tm *, int);
};

cTimeBox::cTimeBox(void)
	: cTextBox(TMBOX_WIDTH + iTimeWidthModifer, TMBOX_HEIGHT, iWhite, iHighlight, fontTimeBox)
{
	SetXAlign(TBOX_TXT_LEFT);
}

cTimeBox::cTimeBox(struct tm *currTime, int offset)
	: cTextBox(TMBOX_WIDTH + iTimeWidthModifer, TMBOX_HEIGHT, iWhite, iHighlight, fontTimeBox)
{
	char timeStr[16];
	time_t t1 = mktime(currTime) + (offset *60);

	(void) localtime_r(&t1, &tmTime);
	if (tmTime.tm_min >= 30) {
		tmTime.tm_min = 30;
	}
	else {
		tmTime.tm_min = 0;
	}
	tmTime.tm_sec = 0;
	hour = tmTime.tm_hour;
	minute = tmTime.tm_min;
		
	SetXAlign(TBOX_TXT_LEFT);
	/* Print time using the configured time format */
	if (YaepgSetup.TimeFormat == TIME_24HR) {
		snprintf(timeStr, 16, "%02d:%02d", hour, minute);
	}
	else {
		snprintf(timeStr, 16, "%d:%02d %s", FMT_12HR(hour), minute,
				FMT_AMPM(hour));
	}
	SetText(1, timeStr);
}

cTimeBox::~cTimeBox()
{
}

int
cTimeBox::SetTime(struct tm *currTime, int offset)
{
	char timeStr[8];
	time_t t1 = mktime(currTime) + (offset *60);

	if (initialized == false) {
		Init(TMBOX_WIDTH + iTimeWidthModifer, TMBOX_HEIGHT, iWhite, iHighlight, fontTimeBox);
	}

	(void) localtime_r(&t1, &tmTime);
	if (tmTime.tm_min > 30) {
		tmTime.tm_min = 30;
	}
	else {
		tmTime.tm_min = 0;
	}
	tmTime.tm_sec = 0;
	hour = tmTime.tm_hour;
	minute = tmTime.tm_min;
		
	SetXAlign(TBOX_TXT_LEFT);
	snprintf(timeStr, 8, "%02d:%02d", hour, minute);
	SetText(1, timeStr);

	return (0);
}

class cEventBox : public cTextBox {
private:
	const cEvent *event;
	int xOff;

public:
	cEventBox(void);
	cEventBox(const cEvent *, time_t);
	~cEventBox();
	const cEvent *GetEvent(void) { return (event); }
	int XOff(void) { return (xOff); }
};

cEventBox::cEventBox(void)
{
}

cEventBox::cEventBox(const cEvent *e, time_t t)
{
	time_t tLen;
	int w, arrowFlag = 0;

	event = e;

	/*
	 * Figure out how long the event is.  The length is adjusted to ensure
	 * that it fits on the screen.
	 */
	tLen = e->Duration();

	if (e->StartTime() < t) {
		arrowFlag |= ARROW_LEFT;
		tLen -= (t - e->StartTime());
	}
	if ((e->StartTime() + e->Duration()) > (t + 7200)) {
		arrowFlag |= ARROW_RIGHT;
		tLen -= ((e->StartTime() + e->Duration()) - (t + 7200));
	}

	/*
	 * We use 5 minute granularity for an event's width. (ie. 5 x 24 = 120 minutes)
	 * We subtract two pixels for the border. 
	 */
	if (tLen >= 300) {
		w = (tLen / 300) * (24 + iEventWidthModifer) - 2;
	}
	else {
		w = 4;
	}

	/*
	 * Calculate the x offset based on the event's start time.  We move it
	 * over one for the left side of the border.  The right border is taken
	 * care of above.
	 *
	 *
	 *  question: if this is X why is it using iOutputResH and not W
	 */
	xOff = (((e->StartTime() - t) + iOutputResH / 2) / 300) * (24 + iEventWidthModifer);
	if (xOff <= 0) {
		xOff = 1;
	}
	else {
		xOff++;
	}

	/*
	 * Now that we have the width, initialize the rest of the fields
	 */
	if (e->HasTimer())
		Init(w, CHBOX_HEIGHT + iEPGMarginY, iHighlight, iLowerBackdrop, fontEventBox);
	else
		Init(w, CHBOX_HEIGHT + iEPGMarginY, iWhite, iBlue, fontEventBox);
	arrow = arrowFlag;
	SetHLColor(iTextHighlighter, iHighlighter);
	SetText(1, e->Title());
}

cEventBox::~cEventBox()
{
}

struct sEventRow {
	int		rowLen;
	cEventBox	*event[8];
};

class cNoInfoEvent : public cEvent {
private:
	time_t		startTime;

public:
	cNoInfoEvent(time_t);
	~cNoInfoEvent();
};

cNoInfoEvent::cNoInfoEvent(time_t startTime)
	: cEvent(0)
{
	SetStartTime(startTime);
	SetDuration(9000);
	SetTitle(tr("No Info"));
	SetDescription(tr("No Info"));
}

cNoInfoEvent::~cNoInfoEvent()
{
}

#define SOURCEBOX_HEIGHT			20

class cSourceBox : public cTextBox {
private:
public:
	cSourceBox(void) {};
	cSourceBox(const cEvent *);
	~cSourceBox() {};
};

cSourceBox::cSourceBox(const cEvent *e)
	:cTextBox(iSrcboxWidth, SOURCEBOX_HEIGHT, iWhite, iTint, fontSourceBox)
{
	if (e == NULL) {
		fprintf(stderr, "e == NULL!\n");
		return;
	}
	if (Channels.GetByChannelID(e->ChannelID())) { 
        cChannel *channel = Channels.GetByChannelID(e->ChannelID());
	SetXAlign(TBOX_TXT_RIGHT);
	SetText(1, *cSource::ToString(channel->Source()));
	}
}

	  /* TIBOX_HEIGHT - DURBOX_HEIGHT - DTBOX_HEIGHT + 2(correct for overlap) */
#define INFBOX_HEIGHT		iOutputResH / 2 - 22 - 19 - 20 + 2    /* TODO */

class cInfoBox : public cTextBox {
private:
	const cEvent		*event;
	void FormatText(void);

public:
	cInfoBox(void);
	cInfoBox(const cEvent *);
	~cInfoBox();
};

cInfoBox::cInfoBox(void)
{
}

cInfoBox::cInfoBox(const cEvent *e)
	: cTextBox(iTxtboxWidth, INFBOX_HEIGHT - iTop, iCyan, iTint, fontInfoBox)
{
	if (e == NULL) {
		fprintf(stderr, "e == NULL!\n");
		return;
	}
	event = e;

	SetYSpacing(2);
	FormatText();
}

cInfoBox::~cInfoBox()
{
}


void
cInfoBox::FormatText(void)
{
        const char *noInfo = tr("No Info");
        const char *startPtr = noInfo;
        char line[iInfboxTxtWidth];
        int breakPos, endPos;
        unsigned int uiDescriptionField = iInfboxTxtWidth;

    if (event->Description() != NULL) {
        startPtr = event->Description();
    }
/*
 *     ** TODO 
 * It might be nice to add a space to the end of the event description
 * somehow so that the last line of text displayed doesn't always
 * usually end up as a single word.
 */

#ifndef MIN
# define MIN(a,b) ((a<b)?a:b)
#endif
   while (*startPtr) {
                endPos = MIN((uiDescriptionField - 1),strlen(startPtr));
                for (breakPos = endPos;
                     !isspace(startPtr[breakPos]) && breakPos;
                     breakPos--);
                /* Space found */
                if (breakPos)
                        endPos = breakPos;

                bzero(line, iInfboxTxtWidth);
                strncpy(line, startPtr, endPos);
        	AddLine(line);
                startPtr = startPtr + endPos;

                /* Skip the space if one found */
                if (breakPos) startPtr++;
        }
}

#define TIBOX_HEIGHT			22

class cTitleBox : public cTextBox {
private:
public:
	cTitleBox(void) {};
	cTitleBox(const cEvent *);
	~cTitleBox() {};
};

cTitleBox::cTitleBox(const cEvent *e)
	:cTextBox(iTxtboxWidth, TIBOX_HEIGHT, iGold, iTint, fontTitleBox)
{
        if ((e->ShortText() == NULL)||(strlen(e->ShortText()) == 0)) {
          SetText(1, e->Title());
        }
        else {
          SetText(1, e->ShortText());
        }

}

#define DURBOX_HEIGHT			20

class cDurBox : public cTextBox {
private:
public:
	cDurBox(void) {};
	cDurBox(const cEvent *);
	~cDurBox() {};
};

cDurBox::cDurBox(const cEvent *e)
	:cTextBox(iTxtboxWidth, DURBOX_HEIGHT, iWhite, iTint, fontDurBox)
{
	char timeStr[80];
	time_t t;
	struct tm startTm, endTm;

	t = e->StartTime();
	(void) localtime_r(&t, &startTm);
	t += e->Duration();
	(void) localtime_r(&t, &endTm);
	if (YaepgSetup.TimeFormat == TIME_24HR) {
		snprintf(timeStr, 80, "%02d:%02d - %02d:%02d (%d min)", startTm.tm_hour,
				startTm.tm_min, endTm.tm_hour, endTm.tm_min, e->Duration() / 60);
	}
	else {
		snprintf(timeStr, 80, "%2d:%02d %s - %2d:%02d %s (%d min)",
				FMT_12HR(startTm.tm_hour), startTm.tm_min,
				FMT_AMPM(startTm.tm_hour),
				FMT_12HR(endTm.tm_hour), endTm.tm_min,
				FMT_AMPM(endTm.tm_hour), e->Duration() / 60);
	}
	SetText(1, timeStr);
}

#ifdef SWITCH_AND_RECORD_PATCH
const char *recFreqStr[6] = {
	"Switch Once",
	"Switch & Record Once",
#else
const char *recFreqStr[4] = {
#endif
	"Record Once",
	"Every",
	"Mon-Fri",
	"Mon-Sun"
};

#define IDBOX_HEIGHT			20

class cIdentBox : public cTextBox {
private:
	struct tm	dateTm;
	time_t		date;

public:
	cIdentBox(void);
	cIdentBox(time_t);
	~cIdentBox();
};

cIdentBox::cIdentBox(void)
{
}

cIdentBox::cIdentBox(time_t time)
	:cTextBox(iIdboxWidth, IDBOX_HEIGHT, iUpperBackdrop, iIdent, fontIdentBox)
{
	char epgStr[16];

	SetXAlign(TBOX_TXT_LEFT);
		snprintf(epgStr, 16, "EPG");
	SetText(1, epgStr);
}

cIdentBox::~cIdentBox()
{
}

#define DTBOX_HEIGHT			20

const char *numToDay[7] = {
	"Sun",
	"Mon",
	"Tue",
	"Wed",
	"Thr",
	"Fri",
	"Sat"
};

class cDateBox : public cTextBox {
private:
	struct tm	dateTm;
	time_t		date;

public:
	cDateBox(void);
	cDateBox(time_t);
	~cDateBox();
};

cDateBox::cDateBox(void)
{
}

cDateBox::cDateBox(time_t time)
	:cTextBox(iDateboxWidth + 2, DTBOX_HEIGHT, iUpperBackdrop, iCyan, fontDateBox)
{
	char dateStr[30];
	date = time;
	(void) localtime_r(&date, &dateTm);

	SetXAlign(TBOX_TXT_RIGHT);
	if (YaepgSetup.TimeFormat == TIME_24HR) {
		snprintf(dateStr, 30, "%s %d.%d %02d:%02d",
				tr(numToDay[dateTm.tm_wday]), dateTm.tm_mday, dateTm.tm_mon+1,
				dateTm.tm_hour, dateTm.tm_min);
	}
	else {
		snprintf(dateStr, 30, "%s %d.%d %02d:%02d %s",
				tr(numToDay[dateTm.tm_wday]), dateTm.tm_mday, dateTm.tm_mon+1,
				FMT_12HR(dateTm.tm_hour), dateTm.tm_min,
				FMT_AMPM(dateTm.tm_hour));
	}
	SetText(1, dateStr);
}

cDateBox::~cDateBox()
{
}

#define RECBOX_WIDTH			410
#define RECBOX_HEIGHT			170

class cRecDlgBox {
private:
	const cEvent		*recEvent;
	cBitmap			*recBmp;
	cTextBox		*recWidgets[10];
	struct tm		recCurrStart;
	struct tm		recCurrStop;
	int			recCurrFreq;
	int			recCursor;
	int			recUseEpName;

public:
	cRecDlgBox(void) {};
	cRecDlgBox(const cEvent *);
	~cRecDlgBox();
	void		Draw(void);
	cBitmap	*GetBmp(void) { return (recBmp); }
	bool		ProcessKey(eKeys);
	bool		AddTimer(void);
};

cRecDlgBox::cRecDlgBox(const cEvent *event)
	: recEvent(event)
{
	time_t tmpT;

	tmpT = recEvent->StartTime() - Setup.MarginStart * 60;
	(void) localtime_r(&tmpT, &recCurrStart);
	tmpT = recEvent->StartTime() + recEvent->Duration() + Setup.MarginStop * 60;
	(void) localtime_r(&tmpT, &recCurrStop);
	/* recBox is best left using a fixed size */
	recCursor 		= 0;


#ifdef SWITCH_AND_RECORD_PATCH
        if (YaepgSetup.DefaultTimer == TIMER_SYSTEM_DEFAULT) {
	   recCurrFreq 		= Setup.DefaultTimerAction;
        }
	else {
#endif
	   recCurrFreq 		= YaepgSetup.DefaultTimer;
#ifdef SWITCH_AND_RECORD_PATCH
        }
#endif
	if (YaepgSetup.DefaultUseEpisode == USE_EPISODE_SMART) {
	   if (recEvent->Duration() <= 80*60 && !isempty(recEvent->ShortText()))
	      recUseEpName = 1;
	   else
	      recUseEpName = 0;
	}
	else if (YaepgSetup.DefaultUseEpisode == USE_EPISODE_ALWAYS) {
	   if (!isempty(recEvent->ShortText()))
	      recUseEpName = 1;
	   else
	      recUseEpName = 0;
	}
	else {
	   recUseEpName = 0;
	}

	recBmp = new cBitmap(RECBOX_WIDTH, RECBOX_HEIGHT, 2);

	recWidgets[0] = new cTextBox(402, 20, iWhite, iLowerBackdrop, fontTimeBox);
	recWidgets[0]->SetXAlign(TBOX_TXT_CENTER);
	recWidgets[0]->SetText(1, recEvent->Title());

	recWidgets[1] = new cTextBox(402, 26, iWhite, iHighlighter, fontTimeBox);
	recWidgets[1]->SetXAlign(TBOX_TXT_CENTER);
	recWidgets[1]->SetText(1, recEvent->ShortText());

	recWidgets[2] = new cTextBox(165, 26, iWhite, iHighlighter, fontTimeBox);
	recWidgets[2]->SetXAlign(TBOX_TXT_LEFT);
	recWidgets[2]->SetText(1, tr("Timer Type:"));

	recWidgets[3] = new cTextBox(235, 26, iWhite, iBlue, fontTimeBox);
	recWidgets[3]->SetXAlign(TBOX_TXT_RIGHT);
	recWidgets[3]->SetHLColor(iTextHighlighter, iHighlighter);

	recWidgets[4] = new cTextBox(165, 26, iWhite, iHighlighter, fontTimeBox);
	recWidgets[4]->SetXAlign(TBOX_TXT_LEFT);
	recWidgets[4]->SetText(1, tr("Use Episode Name:"));

	recWidgets[5] = new cTextBox(235, 26, iWhite, iBlue, fontTimeBox);
	recWidgets[5]->SetXAlign(TBOX_TXT_RIGHT);
	recWidgets[5]->SetHLColor(iTextHighlighter, iHighlighter);

	recWidgets[6] = new cTextBox(165, 26, iWhite, iHighlighter, fontTimeBox);
	recWidgets[6]->SetXAlign(TBOX_TXT_LEFT);
	recWidgets[6]->SetText(1, tr("Start:"));

	recWidgets[7] = new cTextBox(235, 26, iWhite, iBlue, fontTimeBox);
	recWidgets[7]->SetXAlign(TBOX_TXT_RIGHT);
	recWidgets[7]->SetHLColor(iTextHighlighter, iHighlighter);

	recWidgets[8] = new cTextBox(165, 26, iWhite, iHighlighter, fontTimeBox);
	recWidgets[8]->SetXAlign(TBOX_TXT_LEFT);
	recWidgets[8]->SetText(1, tr("Stop:"));

	recWidgets[9] = new cTextBox(235, 26, iWhite, iBlue, fontTimeBox);
	recWidgets[9]->SetXAlign(TBOX_TXT_RIGHT);
	recWidgets[9]->SetHLColor(iTextHighlighter, iHighlighter);
}

cRecDlgBox::~cRecDlgBox()
{
	delete recBmp;
	for (int i = 0; i < 10; i++) {
		delete recWidgets[i];
	}
}

#ifdef SWITCH_AND_RECORD_PATCH
const char *actionList[3] = {
	"Switch",
	"Switch & Record",
#else
const char *actionList[1] = {
#endif
	"Record",
};

void
cRecDlgBox::Draw(void)
{
	char tmpStr[64];
	char epnStr[12];
	int defAction;

#ifdef SWITCH_AND_RECORD_PATCH
        if (YaepgSetup.DefaultTimer == TIMER_SYSTEM_DEFAULT) {
	   defAction 		= Setup.DefaultTimerAction;
        }
	else {
#endif
	   defAction 		= YaepgSetup.DefaultTimer;
#ifdef SWITCH_AND_RECORD_PATCH
        }
#endif
        
	recBmp->DrawRectangle(0, 0, RECBOX_WIDTH, RECBOX_HEIGHT, iWhite);
	recBmp->DrawRectangle(2, 2, RECBOX_WIDTH-3, RECBOX_HEIGHT-3, iLowerBackdrop);

	recWidgets[0]->Draw(4, 4, recBmp);
	recWidgets[1]->Draw(4, 28, recBmp);
	recWidgets[2]->Draw(4, 56, recBmp);
	switch (recCurrFreq) {
	case 0:
#ifdef SWITCH_AND_RECORD_PATCH
	case 1:
	case 2:
#endif
		snprintf(tmpStr, 64, "%s", tr(recFreqStr[recCurrFreq]));
		break;
#ifdef SWITCH_AND_RECORD_PATCH
	case 3:
#else
	case 1:
#endif
		snprintf(tmpStr, 64, "%s %s %s", actionList[defAction], tr(recFreqStr[recCurrFreq]),
				tr(numToDay[recCurrStart.tm_wday]));
		break;
#ifdef SWITCH_AND_RECORD_PATCH
	case 4:
	case 5:
#else
	case 2:
	case 3:
#endif
		snprintf(tmpStr, 64, "%s %s", actionList[defAction], tr(recFreqStr[recCurrFreq]));
		break;
	default:
		break;
	}
	recWidgets[3]->SetText(1, tmpStr);
	if (recCursor == 0) {
		recWidgets[3]->SetHighLight(true, true);
	}
	else {
		recWidgets[3]->SetHighLight(false, false);
	}
	recWidgets[3]->Draw(171, 56, recBmp);
	recWidgets[4]->Draw(4, 84, recBmp);
	switch (recUseEpName) {
	case 0:
		snprintf(epnStr, 12, "No");
		break;
	case 1:
		snprintf(epnStr, 12, "Yes");
		break;	
	default:
		break;
        }
	recWidgets[5]->SetText(1, epnStr);
	if (recCursor == 1) {
		recWidgets[5]->SetHighLight(true, true);
	}
	else {
		recWidgets[5]->SetHighLight(false, false);
	}
	recWidgets[5]->Draw(171, 84, recBmp);
	recWidgets[6]->Draw(4, 112, recBmp);
	if (YaepgSetup.TimeFormat == TIME_24HR) {
		snprintf(tmpStr, 64, "%02d:%02d", recCurrStart.tm_hour,
				recCurrStart.tm_min);
	}
	else {
		snprintf(tmpStr, 64, "%d:%02d %s", FMT_12HR(recCurrStart.tm_hour),
				recCurrStart.tm_min, FMT_AMPM(recCurrStart.tm_hour));
	}
	recWidgets[7]->SetText(1, tmpStr);
	if (recCursor == 2) {
		recWidgets[7]->SetHighLight(true, true);
	}
	else {
		recWidgets[7]->SetHighLight(false, false);
	}
	recWidgets[7]->Draw(171, 112, recBmp);
	recWidgets[8]->Draw(4, 140, recBmp);
	if (YaepgSetup.TimeFormat == TIME_24HR) {
		snprintf(tmpStr, 64, "%02d:%02d", recCurrStop.tm_hour,
				recCurrStop.tm_min);
	}
	else {
		snprintf(tmpStr, 64, "%d:%02d %s", FMT_12HR(recCurrStop.tm_hour),
				recCurrStop.tm_min, FMT_AMPM(recCurrStop.tm_hour));
	}
	recWidgets[9]->SetText(1, tmpStr);
	if (recCursor == 3) {
		recWidgets[9]->SetHighLight(true, true);
	}
	else {
		recWidgets[9]->SetHighLight(false, false);
	}
	recWidgets[9]->Draw(171, 140, recBmp);
}

bool
cRecDlgBox::ProcessKey(eKeys Key)
{
	bool retVal = false;
	time_t t1, t2;

	t1 = mktime(&recCurrStart);
	t2 = mktime(&recCurrStop);

	switch (Key & ~k_Repeat) {
	case kDown:
		if (recCursor <= 3) {
			recCursor++;
			retVal = true;
			if (recCursor > 3)
				recCursor = 0;
		}
		break;

	case kUp:
		if (recCursor >= 0) {
			recCursor--;
			retVal = true;
			if (recCursor < 0)
				recCursor = 3;
		}
		break;

	case kLeft:
		if (recCursor == 1) {
			if (recUseEpName >= 0) {
				if (!isempty(recEvent->ShortText()))
					recUseEpName--;
				retVal = true;
				if (recUseEpName < 0)
					recUseEpName = 1;
			}
		}
		else if (recCursor == 2) {
			t1 -= 60;
			(void) localtime_r(&t1, &recCurrStart);
			retVal = true;
		}
		else if (recCursor == 3) {
			if ((t2 - 60) > t1) {
				t2 -= 60;
				(void) localtime_r(&t2, &recCurrStop);
				retVal = true;
			}
		}
		else {
			if (recCurrFreq >= 0) {
				recCurrFreq--;
				retVal = true;
				if (recCurrFreq < 0)
					recCurrFreq = 4;
			}
		}
		break;

	case kRight:
		if (recCursor == 1) {
			if (recUseEpName <= 1) {
				if (!isempty(recEvent->ShortText()))
					recUseEpName++;
				retVal = true;
				if (recUseEpName > 1)
					recUseEpName = 0;
			}
		}
		else if (recCursor == 2) {
			if ((t1 + 60) < t2) {
				t1 += 60;
				(void) localtime_r(&t1, &recCurrStart);
				retVal = true;
			}
		}
		else if (recCursor == 3) {
			t2 += 60;
			(void) localtime_r(&t2, &recCurrStop);
			retVal = true;
		}
		else {
			if (recCurrFreq <= 4) {
				recCurrFreq++;
				retVal = true;
				if (recCurrFreq > 4)
					recCurrFreq = 0;
			}
		}
		break;

	default:
		retVal = false;
		break;
	}

	return (retVal);
}

static const char *wdayToFreq[7] = {
	"------S",
	"M------",
	"-T-----",
	"--W----",
	"---T---",
	"----F--",
	"-----S-",
};

bool
cRecDlgBox::AddTimer(void)
{
	cTimer *recTimer;
	int flags, channel, start, stop, priority, lifetime, action;
        char dayStr[8], file[MaxFileName], eventStr[256];

	/* Create the timer */
	recTimer = new cTimer;

	/* Construct the string that represents the event */
	flags = tfActive;
	channel = Channels.GetByChannelID(recEvent->ChannelID(), true)->Number();
	start = (recCurrStart.tm_hour * 100) + recCurrStart.tm_min;
	stop = (recCurrStop.tm_hour * 100) + recCurrStop.tm_min;
	priority = Setup.DefaultPriority;
	lifetime = Setup.DefaultLifetime;
	*file = '\0';
	if (!isempty(recEvent->Title())) {
		if (recUseEpName == 1 ) {
		   strncpy(file, recEvent->Title(), sizeof(file));
		   strcat(file, "~");
		   strcat(file, recEvent->ShortText());
		   for (int checkchar = 0; checkchar < sizeof(file); checkchar++) {
			if (file[checkchar] == ':') {
			   file[checkchar] = '|';
			}
		   }
		}
		else {
		   strncpy(file, recEvent->Title(), sizeof(file));
		}
	}
#ifdef SWITCH_AND_RECORD_PATCH
	switch (recCurrFreq) {
	case 0:
		snprintf(dayStr, 8, "%d", recCurrStart.tm_mday);
		action = TIMER_SWITCH;
		break;
	case 1:
		snprintf(dayStr, 8, "%d", recCurrStart.tm_mday);
		action = TIMER_SWITCH_AND_RECORD;
		break;	
	case 2:
		snprintf(dayStr, 8, "%d", recCurrStart.tm_mday);
		action = TIMER_RECORD;
		break;
	case 3:
		sprintf(dayStr, "%s", wdayToFreq[recCurrStart.tm_wday]);
		action = Setup.DefaultTimerAction;
		break;
	case 4:
		sprintf(dayStr, "MTWTF--");
		action = Setup.DefaultTimerAction;
		break;
	case 5:
		sprintf(dayStr, "MTWTFSS");
		action = Setup.DefaultTimerAction;
		break;
#else
	action = TIMER_RECORD;
	switch (recCurrFreq) {
	case 0:
		snprintf(dayStr, 8, "%d", recCurrStart.tm_mday);
		break;
	case 1:
		sprintf(dayStr, "%s", wdayToFreq[recCurrStart.tm_wday]);
		break;
	case 2:
		sprintf(dayStr, "MTWTF--");
		break;
	case 3:
		sprintf(dayStr, "MTWTFSS");
		break;
#endif
	default:
		delete recTimer;
		return false;
	}
	snprintf(eventStr, 256, "%d:%d:%s:%04d:%04d:%d:%d:%d:%s:",
			flags, channel, dayStr, start, stop,
			priority, lifetime, action, file);
	/* fprintf(stderr, "EVENT: %s\n", eventStr); */

	/* Parse the string to fill in the timer */
	if (recTimer->Parse(eventStr) == false) {
		delete recTimer;
		return false;
	}

	/* Add the timer to the timer list and flush to disk */
	Timers.Add(recTimer);
	Timers.Save();
	/* Perform a timers load so that the system becomes aware of the new timer right away */
	for (cTimer *timer = Timers.First(); timer; timer = Timers.Next(timer)) {
      		timer->SetEventFromSchedule();
	}

	return true;
}

#define MSGBOX_WIDTH				360
#define MSGBOX_HEIGHT				42

class cMessageBox {
private:
	cBitmap		*msgBmp;
	cTextBox	*msgBox;

public:
	cMessageBox(const char *);
	~cMessageBox();
	void		Draw(void);
	cBitmap	*GetBmp(void) { return (msgBmp); }
};

cMessageBox::cMessageBox(const char *msg)
{
	msgBmp = new cBitmap(MSGBOX_WIDTH, MSGBOX_HEIGHT, 2);

	msgBox = new cTextBox(MSGBOX_WIDTH - 8, MSGBOX_HEIGHT - 8, iWhite, iBlue, fontTimeBox);
	msgBox->SetXAlign(TBOX_TXT_CENTER);
	msgBox->SetText(1, msg);
}

cMessageBox::~cMessageBox()
{
	/* delete msgBmp; */
	/* delete msgBox; */
}

void
cMessageBox::Draw(void)
{
	msgBmp->DrawRectangle(0, 0, MSGBOX_WIDTH, MSGBOX_HEIGHT, iWhite);
	msgBmp->DrawRectangle(2, 2, MSGBOX_WIDTH-3, MSGBOX_HEIGHT-3, iLowerBackdrop);
	msgBox->Draw(4, 4, msgBmp);
}

class cMsgDlgBox {
private:
	cBitmap		*msgDlgBmp;
	cTextBox	*msgDlgBox;

public:
	cMsgDlgBox(const char *);
	~cMsgDlgBox();
	void		Draw(void);
	cBitmap	*GetBmp(void) { return (msgDlgBmp); }
};

cMsgDlgBox::cMsgDlgBox(const char *msg)
{
	msgDlgBmp = new cBitmap(MSGBOX_WIDTH, MSGBOX_HEIGHT, 2);

	msgDlgBox = new cTextBox(MSGBOX_WIDTH - 8, MSGBOX_HEIGHT - 8, iWhite, iBlue, fontTimeBox);
	msgDlgBox->SetXAlign(TBOX_TXT_CENTER);
	msgDlgBox->SetText(1, msg);
}

cMsgDlgBox::~cMsgDlgBox()
{
	/* delete msgDlgBmp; */
	/* delete msgDlgBox; */
}

void
cMsgDlgBox::Draw(void)
{
	msgDlgBmp->DrawRectangle(0, 0, MSGBOX_WIDTH, MSGBOX_HEIGHT, iWhite);
	msgDlgBmp->DrawRectangle(2, 2, MSGBOX_WIDTH-3, MSGBOX_HEIGHT-3, iLowerBackdrop);
	msgDlgBox->Draw(4, 4, msgDlgBmp);
}

#define CURSOR_UP			0
#define CURSOR_DOWN			1
#define CURSOR_LEFT			2
#define CURSOR_RIGHT			3

#define SCROLL_LEFT			1
#define SCROLL_RIGHT			2

class cYaepg: public cOsdObject {
private:
	cOsd			*osd;
	tArea			areas[3];
	cBitmap			*EPGBitmap;
	cBitmap			*InfoBitmap;

	int			cursorX;
	int			cursorY;
	int			scrollDir;
	unsigned short		prevEventId;

	/*
	 * EPG window stuff.
	 */
	cTimeBox		*timebox[4];
	cChanBox		*chans[6];    //TODO
	sEventRow		eventgrid[6]; //TODO
        
	int                     last_channel, first_channel;
	
	/*
	 * Info window stuff.
	 */
	cDurBox			*durbox;
	cTitleBox		*titlebox;
	cInfoBox		*infobox;
	cDateBox		*datebox;
	cIdentBox		*identbox;
	cSourceBox		*sourcebox;

	cRecDlgBox		*recDlgBox;
	cMsgDlgBox		*msgDlgBox;
	uint64_t		msgboxStart;
	cMessageBox		*msgbox;

	/*
	 * Times:
	 * 	first - The earliest time that can be displayed in the grid.
	 * 	start - The earliest time that is currently displayed in the grid.
	 * 	curr - The current time.
	 * 	lastup - The last time the display was updated.
	 */
	time_t			firstT;
	struct tm		firstTime;
	time_t			startT;
	struct tm		startTime;
	time_t			currT;
	struct tm		currTime;
	time_t			lastupT;
	int			startChan;

	int			selChan;
	const cEvent		*selEvent;

	/*
	 * Direct Channel Change
	 */
	int			currChan;
	uint64_t		lastInput;

	/*
	 * The last possible starting channel for the grid.
	 */
	int			lastChan;

	/*
	 * Debugging
	 */
	static FILE		*log;

	/*
	 * Round a time down to a 30-minute boundary.
	 */
	time_t AdjustTime(time_t);

public:
	cYaepg(void);
	~cYaepg();
	virtual void Show(void);
	virtual eOSState ProcessKey(eKeys);
	void Draw(void);
	void SwitchToCurrentChannel(void);
	void MoveCursor(int);
	void ChangeChan(int);
	int ChangeTime(int);
	void GetChannelListEnds(void);
	void UpdateTime(void);
	static void LogMsg(eYlog_t, const char *fmt, ...);
};

FILE *cYaepg::log = NULL;

cYaepg::cYaepg(void)
{
#ifdef XINE_UI_ZOOM_PATCH
        FILE *zoom_info_output, *deinterlace_info_output;
#endif

	int h;
	int i = 1;
	int j;
	int different;
	int currentSource, curSource;
	int channelSource;

	cChannel *currentChannel;

	osd = NULL;
	EPGBitmap = NULL;
	InfoBitmap = NULL;

	
	cursorX	= 0;
	cursorY = 0;
	scrollDir = 0;
	prevEventId = 0;

	iShowAll = 1;
#ifdef FAVORITES_AND_SORT_PATCH
	iHasFavorites = 0;
        iShowFavorites = 0;
#endif
	iChanSource = 0;
	iLastSource = 0;
	selChan = 0;

	log = fopen("/var/log/yaepg.log", "w+");
	if (log == NULL) {
		fprintf(stderr, "Could not open log file!\n");
	}

#ifdef XINE_UI_ZOOM_PATCH
	/* check zoom */
        zoom_info_output = popen("xine --session zoom_info", "r");
        if (!zoom_info_output) {
        	printf("couldn't get previous zoom state\n");
		strcpy(cPreviousZoom,"");  /* clear filename */
		sprintf(cPreviousZoom, "0");
        }
        else {
       		char* zoom_result = fgets(cPreviousZoom, 8, zoom_info_output);
	}
        pclose(zoom_info_output);
        if (system("xine --session zoom_reset")) {
          /* reset video to 1:1 */
        }

	if (YaepgSetup.ToggleInterlace == true) {
		/* check deinterlace */
        	deinterlace_info_output = popen("xine --session interleave_info", "r");
        	if (!deinterlace_info_output) {
        		printf("couldn't get previous deinterlace state\n");
			strcpy(cPreviousDeinterlace,"");  /* clear filename */
			sprintf(cPreviousDeinterlace, "0");
        	}
        	else {
       			char* deinterlace_mode = fgets(cPreviousDeinterlace, sizeof(cPreviousDeinterlace), deinterlace_info_output);
		}
        	pclose(deinterlace_info_output);
        	iDeinterlaceState = atoi(cPreviousDeinterlace); /* convert to int */
		if (iDeinterlaceState == 1) {
			if (system("xine --session interleave")) {
			  /* toggle interlace mode */
			}
		}
	}
#endif

	/* find all channel sources and make a list */
	currentChannel = Channels.GetByNumber(cDevice::CurrentChannel());
	currentSource = currentChannel->Source();

	for (h = 0; h < TOTALSOURCES; h++) {
		source_list[h] = -1;
	}
	source_list[0] = currentSource;

	for (cChannel *channel = Channels.First(); channel; channel = Channels.Next(channel)) {
		channelSource = channel->Source();

#ifdef FAVORITES_AND_SORT_PATCH
		if (channel->IsFavorite())
			iHasFavorites = 1; /* if user has a favorites list - enable "sort by favorite" */
#endif

		if ((channel->Source() != source_list[i]) && !channel->GroupSep()) {
			different = 1;
			for (j = i-1; j >= 0; j--) {
				if (source_list[j] == channel->Source())
					different = 0;
			}
			if (different == 1) {
				source_list[i] = channel->Source();
				i++;
			}
		}
	}
	for (curSource = 0; curSource < TOTALSOURCES; curSource++) {
		if (source_list[curSource] == -1) {
			iLastSource = curSource;
		break;
		}
	}
}

cYaepg::~cYaepg()
{
	if (osd != NULL) {
		delete osd;
	}
	if (EPGBitmap != NULL) {
		delete EPGBitmap;
	}
	if (InfoBitmap != NULL) {
		delete InfoBitmap;
	}
#ifdef XINE_UI_ZOOM_PATCH
	if (iZoomToPrevious == 1) {
		if (strncmp(cPreviousZoom, "0", 1) == 0) {
			if (system("xine --session zoom_reset")) {
			  /* clear black bar artifacts if they exist */
			}
		}
		else {
			char cZoomCommand[80];
			strcpy(cZoomCommand,"");  /* clear filename */
			sprintf(cZoomCommand, "xine --session zoom=%s\n", cPreviousZoom);
			if (system(cZoomCommand)) {
			  /* zoom to previous value */
			}
		}
	}
	else {
		if (system("xine --session zoom_reset")) {
		  /* clear black bar artifacts if they exist */
		}
	}
	if (YaepgSetup.ToggleInterlace == true) {
		if (iDeinterlaceState == 1) {
			if (system("xine --session interleave")) {
			  /* toggle interlace mode */
			}
		}
	}
#endif
	if (log) {
		fflush(log);
		fclose(log);
	}
}

void
cYaepg::Show(void)
{
	osd = cOsdProvider::NewOsd(0, 0);
	if (osd == NULL) {
		fprintf(stderr, "OpenRaw() returned NULL!\n");
		return;
	}
	if (YaepgSetup.TvFormat == TV_PAL) {
		iOutputResW	= TV_PAL_W;
		iOutputResH	= TV_PAL_H;
		iChannelCount	= CHANNELS_PAL;
	}
	else {
		iOutputResW	= TV_NTSC_W;
		iOutputResH	= TV_NTSC_H;
		iChannelCount	= CHANNELS_NTSC;
	}
	iEPGMarginX				= 0;
	/*
	 * As you can see below there is a bit of duplicate code, but
	 * this has been set up as a case statement to improve speed
	 * by only assigning the values once as it jumps in and breaks out.
	 * Much better then a long set of if's
	 */
	switch (YaepgSetup.SkinStyle) {
	case SKIN_TRANSPARENT:
		/* colors */
		switch (YaepgSetup.Theme) {
		case DEFAULT_THEME:
			iUpperBackdrop		= SKIN_TRANSPARENT_TINT;
			iIdent 			= SKIN_SOLID_CYAN;
			iWhite			= SKIN_SOLID_WHITE;
			iGold			= SKIN_SOLID_GOLD;
			iCyan			= SKIN_SOLID_CYAN;
			iTint			= SKIN_TRANSPARENT_TINT;
			iLowerBackdrop		= SKIN_TRANSPARENT_BACKDROP;
			iBlue			= SKIN_TRANSPARENT_BLUE;
			iHighlight 		= SKIN_TRANSPARENT_HIGHLIGHT;
			iHighlighter 		= iHighlight;
			iTextHighlighter 	= iWhite;
		break;
		case CONTRAST_THEME:
			iUpperBackdrop		= SKIN_TRANSPARENT_TINT;
			iIdent 			= SKIN_SOLID_CYAN;
			iWhite			= SKIN_SOLID_WHITE;
			iGold			= SKIN_SOLID_GOLD;
			iCyan			= SKIN_SOLID_CYAN;
			iTint			= SKIN_TRANSPARENT_TINT;
			iLowerBackdrop		= SKIN_TRANSPARENT_BACKDROP;
			iBlue			= SKIN_TRANSPARENT_BLUE;
			iHighlight 		= SKIN_TRANSPARENT_ALT_HIGHLIGHT;
			iHighlighter 		= iHighlight;
			iTextHighlighter 	= iWhite;
		break;
		case THEATER_THEME:
			iUpperBackdrop		= SKIN_TRANSPARENT_TINT;
			iIdent 			= 0xFF57BCD9;
			iWhite			= 0xFFC2C6C6;
			iGold			= 0xFFCCBB22;
			iCyan			= 0xFF57BCD9;
			iTint			= SKIN_TRANSPARENT_TINT;
			iLowerBackdrop		= SKIN_TRANSPARENT_BACKDROP;
			iBlue			= 0xCD05173A;
			iHighlight 		= 0xCA2D3D50;
			iHighlighter 		= iHighlight;
			iTextHighlighter 	= iWhite;
		break;
		}
		/* dimensions */
		if (YaepgSetup.ScreenMode == MODE_FULLSCREEN) {
			iTop			= 0;
			iChanWidthModifer	= 18;
			iTimeWidthModifer	= 12;
			iEventWidthModifer	= 2;
			iEPGMarginY		= 5;
			iEPGMainMarginY		= 0;
			iLeftMarginLimit	= 1;
			iIdboxWidth		= iOutputResW / 2;
			iDateboxWidth		= iOutputResW - iIdboxWidth;
			iInfboxTxtWidth 	= 64;
		}
		else {
			iTop			= 30;
			iChanWidthModifer	= 0;
			iTimeWidthModifer	= 0;
			iEventWidthModifer	= 0;
			iEPGMarginY		= 0;
			iEPGMainMarginY		= ((iChannelCount * 5) + 3);
			iLeftMarginLimit	= MAX_LEFT_MARGIN;
			iIdboxWidth		= iOutputResW / 2 - iLeftMarginLimit - 14;
			iDateboxWidth		= iOutputResW - iLeftMarginLimit - iIdboxWidth;
			iInfboxTxtWidth 	= 60;
		}
		iInfboxWidth			= iOutputResW - iLeftMarginLimit;
		iInfboxOffset 			= YaepgSetup.LeftMargin + 4 + 7;
		iTxtboxWidth			= iInfboxWidth - iInfboxOffset + 4 + 14;
	break;
	case SKIN_SOLID:
		/* colors */
		switch (YaepgSetup.Theme) {
		case DEFAULT_THEME:
			iUpperBackdrop		= SKIN_SOLID_BACKDROP;
			iIdent 			= SKIN_SOLID_CYAN;
			iWhite			= SKIN_SOLID_WHITE;
			iGold			= SKIN_SOLID_GOLD;
			iCyan			= SKIN_SOLID_CYAN;
			/*
			 * Setting tint to border instead of clear makes gold disappear
			 * reason: only 3 colors per surface (gold is drawn last)
			 */
			iTint			= SKIN_SOLID_BACKDROP;
			iLowerBackdrop		= SKIN_SOLID_BACKDROP;
			iBlue			= SKIN_SOLID_BLUE;
			iHighlight 		= SKIN_SOLID_HIGHLIGHT;
			iHighlighter 		= iHighlight;
			iTextHighlighter 	= iWhite;
		break;
		case CONTRAST_THEME:
			iUpperBackdrop		= SKIN_SOLID_BACKDROP;
			iIdent 			= SKIN_SOLID_CYAN;
			iWhite			= SKIN_SOLID_WHITE;
			iGold			= SKIN_SOLID_GOLD;
			iCyan			= SKIN_SOLID_CYAN;
			/*
			 * Setting tint to border instead of clear makes gold disappear
			 * reason: only 3 colors per surface (gold is drawn last)
			 */
			iTint			= SKIN_SOLID_BACKDROP;
			iLowerBackdrop		= SKIN_SOLID_BACKDROP;
			iBlue			= SKIN_SOLID_BLUE;
			iHighlight 		= SKIN_SOLID_ALT_HIGHLIGHT;
			iHighlighter 		= iHighlight;
			iTextHighlighter 	= iWhite;
		break;
		case THEATER_THEME:
			iUpperBackdrop		= SKIN_SOLID_BACKDROP;
			iIdent 			= 0xFF57BCD9;
			iWhite			= 0xFFC2C6C6;
			iGold			= 0xFFCCBB22;
			iCyan			= 0xFF57BCD9;
			/*
			* Setting tint to border instead of clear makes gold disappear
		 	* reason: only 3 colors per surface (gold is drawn last)
		 	*/
			iTint			= SKIN_SOLID_BACKDROP;
			iLowerBackdrop		= SKIN_SOLID_BACKDROP;
			iBlue			= 0xFF05183B;
			iHighlight 		= 0xFF2D3D50;
			iHighlighter 		= iHighlight;
			iTextHighlighter 	= iWhite;
		break;
		}
		/* dimensions */
		if (YaepgSetup.ScreenMode == MODE_FULLSCREEN) {
			iTop			= 0;
			iChanWidthModifer	= 18;
			iTimeWidthModifer	= 12;
			iEventWidthModifer	= 2;
			iEPGMarginY		= 5;
			iEPGMainMarginY		= 0;
			iLeftMarginLimit	= 1;
			iIdboxWidth		= iOutputResW / 2;
			iDateboxWidth		= iOutputResW - iIdboxWidth;
			iInfboxTxtWidth 	= 64;
		}
		else {
			iTop			= 30;
			iChanWidthModifer	= 0;
			iTimeWidthModifer	= 0;
			iEventWidthModifer	= 0;
			iEPGMarginY		= 0;
			iEPGMainMarginY		= ((iChannelCount * 5) + 3);
			iLeftMarginLimit	= MAX_LEFT_MARGIN;
			iIdboxWidth		= iOutputResW / 2 - iLeftMarginLimit - 14;
			iDateboxWidth		= iOutputResW - iLeftMarginLimit - iIdboxWidth;
			iInfboxTxtWidth 	= 60;
		}
		iInfboxWidth			= iOutputResW - iLeftMarginLimit;
		iInfboxOffset 			= YaepgSetup.LeftMargin + 4 + 7;
		iTxtboxWidth			= iInfboxWidth - iInfboxOffset + 4 + 14;
	break;
	case SKIN_WINDOWED:
		/* colors */
		switch (YaepgSetup.Theme) {
		case DEFAULT_THEME:
			iUpperBackdrop		= SKIN_TRANSPARENT_CLEAR;
			iIdent 			= SKIN_SOLID_CYAN;
			iWhite			= SKIN_SOLID_WHITE;
			iGold			= SKIN_SOLID_GOLD;
			iCyan			= SKIN_SOLID_CYAN;
			/*
			 * Setting tint to border instead of clear makes gold disappear
			 * reason: only 3 colors per surface (gold is drawn last)
			 */
			iTint			= SKIN_SOLID_BACKDROP;
			iLowerBackdrop		= SKIN_SOLID_BACKDROP;
			iBlue			= SKIN_SOLID_BLUE;
			iHighlight 		= SKIN_SOLID_HIGHLIGHT;
			iHighlighter 		= iHighlight;
			iTextHighlighter 	= iWhite;
		break;
		case CONTRAST_THEME:
			iUpperBackdrop		= SKIN_TRANSPARENT_CLEAR;
			iIdent 			= SKIN_SOLID_CYAN;
			iWhite			= SKIN_SOLID_WHITE;
			iGold			= SKIN_SOLID_GOLD;
			iCyan			= SKIN_SOLID_CYAN;
			/*
			 * Setting tint to border instead of clear makes gold disappear
			 * reason: only 3 colors per surface (gold is drawn last)
			 */
			iTint			= SKIN_SOLID_BACKDROP;
			iLowerBackdrop		= SKIN_SOLID_BACKDROP;
			iBlue			= SKIN_SOLID_BLUE;
			iHighlight 		= SKIN_SOLID_ALT_HIGHLIGHT;
			iHighlighter 		= iHighlight;
			iTextHighlighter 	= iWhite;
		break;
		case THEATER_THEME:
			iUpperBackdrop		= SKIN_TRANSPARENT_CLEAR;
			iIdent 			= 0xFF57BCD9;
			iWhite			= 0xFFC2C6C6;
			iGold			= 0xFFCCBB22;
			iCyan			= 0xFF57BCD9;
			/*
			* Setting tint to border instead of clear makes gold disappear
		 	* reason: only 3 colors per surface (gold is drawn last)
		 	*/
			iTint			= SKIN_SOLID_BACKDROP;
			iLowerBackdrop		= SKIN_SOLID_BACKDROP;
			iBlue			= 0xFF05183B;
			iHighlight 		= 0xFF2D3D50;
			iHighlighter 		= iHighlight;
			iTextHighlighter 	= iWhite;
		break;
		}
		/* dimensions */
		if (YaepgSetup.ScreenMode == MODE_FULLSCREEN) {
			iTop			= 0;
			iChanWidthModifer	= 18;
			iTimeWidthModifer	= 12;
			iEventWidthModifer	= 2;
			iEPGMarginY		= 5;
			iEPGMainMarginY		= 0;
			iLeftMarginLimit	= 1;
			iIdboxWidth		= iOutputResW / 2 - 30;
			iInfboxOffset		= iIdboxWidth + 4;
			iDateboxWidth		= iOutputResW - iIdboxWidth;
			iInfboxTxtWidth 	= 40;
		}
		else {
			iTop			= 30;
			iChanWidthModifer	= 0;
			iTimeWidthModifer	= 0;
			iEventWidthModifer	= 0;
			iEPGMarginY		= 0;
			iEPGMainMarginY		= ((iChannelCount * 5) + 3);
			iLeftMarginLimit	= MAX_LEFT_MARGIN;
			iIdboxWidth		= iOutputResW / 2 - iLeftMarginLimit - 14;
			iInfboxOffset		= iIdboxWidth + YaepgSetup.LeftMargin + 7 + 2;
			iDateboxWidth		= iOutputResW - iLeftMarginLimit - iIdboxWidth;
			iInfboxTxtWidth 	= 36;
		}
		iInfboxWidth			= iOutputResW - iLeftMarginLimit;
		iTxtboxWidth			= iDateboxWidth + 4;
		/* create a video window */
		osd->vidWin.x1			= YaepgSetup.LeftMargin + 2;
		osd->vidWin.y1			= iTop + DTBOX_HEIGHT + 2;
		osd->vidWin.x2			= iInfboxOffset - 2;
		osd->vidWin.y2			= iOutputResH / 2 - 1;
		osd->vidWin.bpp 		= 12;
	break;
	case SKIN_WINDOWED_LARGE:
		/* colors */
		switch (YaepgSetup.Theme) {
		case DEFAULT_THEME:
			iUpperBackdrop		= SKIN_TRANSPARENT_CLEAR;
			iIdent 			= SKIN_TRANSPARENT_CLEAR;
			iWhite			= SKIN_SOLID_WHITE;
			iGold			= SKIN_SOLID_GOLD;
			iCyan			= SKIN_SOLID_CYAN;
			/*
			* Setting tint to border instead of clear makes gold disappear
		 	* reason: only 3 colors per surface (gold is drawn last)
		 	*/
			iTint			= SKIN_SOLID_BACKDROP;
			iLowerBackdrop		= SKIN_SOLID_BACKDROP;
			iBlue			= SKIN_SOLID_BLUE;
			iHighlight 		= SKIN_SOLID_HIGHLIGHT;
			iHighlighter 		= iHighlight;
			iTextHighlighter 	= iWhite;
		break;
		case CONTRAST_THEME:
			iUpperBackdrop		= SKIN_TRANSPARENT_CLEAR;
			iIdent 			= SKIN_TRANSPARENT_CLEAR;
			iWhite			= SKIN_SOLID_WHITE;
			iGold			= SKIN_SOLID_GOLD;
			iCyan			= SKIN_SOLID_CYAN;
			/*
			 * Setting tint to border instead of clear makes gold disappear
			 * reason: only 3 colors per surface (gold is drawn last)
			 */
			iTint			= SKIN_SOLID_BACKDROP;
			iLowerBackdrop		= SKIN_SOLID_BACKDROP;
			iBlue			= SKIN_SOLID_BLUE;
			iHighlight 		= SKIN_SOLID_ALT_HIGHLIGHT;
			iHighlighter 		= iHighlight;
			iTextHighlighter 	= iWhite;
		break;
		case THEATER_THEME:
			iUpperBackdrop		= SKIN_TRANSPARENT_CLEAR;
			iIdent 			= SKIN_TRANSPARENT_CLEAR;
			iWhite			= 0xFFC2C6C6;
			iGold			= 0xFFCCBB22;
			iCyan			= 0xFF57BCD9;
			/*
			* Setting tint to border instead of clear makes gold disappear
		 	* reason: only 3 colors per surface (gold is drawn last)
		 	*/
			iTint			= SKIN_SOLID_BACKDROP;
			iLowerBackdrop		= SKIN_SOLID_BACKDROP;
			iBlue			= 0xFF05183B;
			iHighlight 		= 0xFF2D3D50;
			iHighlighter 		= iHighlight;
			iTextHighlighter 	= iWhite;
		break;
		}
		/* dimensions */
		if (YaepgSetup.ScreenMode == MODE_FULLSCREEN) {
			iTop			= 0;
			iChanWidthModifer	= 18;
			iTimeWidthModifer	= 12;
			iEventWidthModifer	= 2;
			iEPGMarginY		= 5;
			iEPGMainMarginY		= 0;
			iLeftMarginLimit	= 1;
			iIdboxWidth		= iOutputResW / 2;
			iDateboxWidth		= iOutputResW - iIdboxWidth;
			iInfboxTxtWidth 	= 40;
		}
		else {
			iTop			= 30;
			iChanWidthModifer	= 0;
			iTimeWidthModifer	= 0;
			iEventWidthModifer	= 0;
			iEPGMarginY		= 0;
			iEPGMainMarginY		= ((iChannelCount * 5) + 3);
			iLeftMarginLimit	= MAX_LEFT_MARGIN;
			iIdboxWidth		= iOutputResW / 2 - iLeftMarginLimit - 14;
			iDateboxWidth		= iOutputResW - iLeftMarginLimit - iIdboxWidth;
			iInfboxTxtWidth 	= 36;
		}
		iInfboxWidth			= iOutputResW - iLeftMarginLimit;
		iInfboxOffset			= iIdboxWidth + YaepgSetup.LeftMargin;
		iTxtboxWidth			= iDateboxWidth + 4;
		/* create a video window */
		osd->vidWin.x1			= 0;
		osd->vidWin.y1			= 0;
		osd->vidWin.x2			= iInfboxOffset - 2;
		osd->vidWin.y2			= iOutputResH / 2 - 1;
		osd->vidWin.bpp 		= 12;
	break;
	default:
	break;
	}
		iSrcboxWidth			= 115;

	if (YaepgSetup.ScreenMode == MODE_CONDENSED) {
		/* create the EPG window   (border) */
		EPGWinHandle.x1  = 0;
		EPGWinHandle.y1  = iOutputResH / 2;
		EPGWinHandle.x2  = iOutputResW - 1;
		EPGWinHandle.y2  = iOutputResH - 1;
		EPGWinHandle.bpp = 2;
		EPGBitmap = new cBitmap(iOutputResW, iOutputResH / 2, 2);

		/* create the info window   (backdrop) */
		InfoWinHandle.x1  = 0;
		InfoWinHandle.y1  = 0;
		InfoWinHandle.x2  = iOutputResW - 1;
		InfoWinHandle.y2  = iOutputResH / 2 - 1;
		InfoWinHandle.bpp = 2;
	}
	else {
		/* create the EPG window   (border) */
		EPGWinHandle.x1  = YaepgSetup.LeftMargin;
		EPGWinHandle.y1  = iOutputResH / 2;
		EPGWinHandle.x2  = iOutputResW + YaepgSetup.LeftMargin - iLeftMarginLimit;
		EPGWinHandle.y2  = iOutputResH - iEPGMainMarginY - 1;
		EPGWinHandle.bpp = 2;
		EPGBitmap = new cBitmap(iOutputResW, iOutputResH / 2, 2);

		/* create the info window   (backdrop) */
		InfoWinHandle.x1  = YaepgSetup.LeftMargin;
		InfoWinHandle.y1  = iTop;
		InfoWinHandle.x2  = iOutputResW + YaepgSetup.LeftMargin - iLeftMarginLimit;
		InfoWinHandle.y2  = iOutputResH / 2 - 1;
		InfoWinHandle.bpp = 2;
	}

	osd->SetAreas(areas, 2);
	InfoBitmap = new cBitmap(iOutputResW, iOutputResH / 2, 2);


	(void) time(&currT);
	(void) localtime_r(&currT, &currTime);

	firstT = startT = AdjustTime(currT);
	(void) localtime_r(&firstT, &firstTime);
	(void) localtime_r(&startT, &startTime);

	startChan = cDevice::CurrentChannel();
	currChan  = 0;
	recDlgBox = NULL;
	msgDlgBox = NULL;
	msgbox    = NULL;

	/*
	 * The maximum start channel is the last channel - the number of
	 * channels displayed in the grid.  Ensure that this isn't violated, and
	 * adjust the initial cursor position as needed.
	 */
	cChannel *c = Channels.Last();
	YAEPG_INFO("Channels.Last() = %d\n", c->Number());
	for (int i = 0; i < (iChannelCount - 1); i++) {
		while ((c = (cChannel *)c->Prev()) && (c->GroupSep())) {
			;
		}
	}
	if (c == NULL) {
		YAEPG_ERROR("Can't find %d valid channels!\n", iChannelCount);
		return;
	}
	lastChan = c->Number();
	YAEPG_INFO("lastChan = %d\n", lastChan);

	Draw();
}

void
cYaepg::SwitchToCurrentChannel(void)
{
	if (selChan != cDevice::CurrentChannel()) {
		/*
		 * The "Channel not available message" will
		 * cause vdr to crash. Do a "lower level"
		 * channel switch to avoid the error message.
		 */
		const cChannel *c = Channels.GetByNumber(selChan);
		eSetChannelResult ret;

		if (c != NULL) {
			ret = cDevice::PrimaryDevice()->SetChannel(c, true);
			if (ret != scrOk) {
				fprintf(stderr, "SetChannel(): %d\n", ret);
			}
		}
	}
	return;	
}

void
cYaepg::GetChannelListEnds()
{
        /* Set the first and last channel numbers */
        cChannel *cF = Channels.First();
        cChannel *cL = Channels.Last();
        first_channel = cF->Number();
        last_channel = cL->Number();
#ifdef FAVORITES_AND_SORT_PATCH
        iSrcHasFavorite = 0;
#endif

	if (iShowAll != 1) {
#ifdef FAVORITES_AND_SORT_PATCH
		/* first see if current source has a favorite */
		for (cFavorite *favorite = Favorites.First(); favorite; favorite = Favorites.Next(favorite)) {
			if ((favorite->Channel())->Source() == source_list[iChanSource]) {
				iSrcHasFavorite = 1;
			}
		}
		if (iSrcHasFavorite != 1)
			iShowFavorites = 0;

		if (iShowFavorites == 1) {
			while ((Channels.GetByNumber(first_channel)->Source() != source_list[iChanSource]) && (!cF->IsFavorite()) && (cF->GroupSep())) {
				cF = Channels.Next(cF);
			}
			while ((Channels.GetByNumber(last_channel)->Source() != source_list[iChanSource]) && (!cL->IsFavorite()) && (cL->GroupSep())) {
				cL = Channels.Prev(cL);
			}
		} else {
#endif
			while ((Channels.GetByNumber(first_channel)->Source() != source_list[iChanSource]) && (cF->GroupSep())) {
				cF = Channels.Next(cF);
			}
			while ((Channels.GetByNumber(last_channel)->Source() != source_list[iChanSource]) && (cL->GroupSep())) {
				cL = Channels.Prev(cL);
				last_channel = cL->Number();
			}
#ifdef FAVORITES_AND_SORT_PATCH
		}
#endif
	}
}

void
cYaepg::Draw(void)
{
	cChannel *c;
	int i, j, cx, cy;

	/*
	 * Draw the EPG window
	 */
	EPGBitmap->DrawRectangle(0, 0, iOutputResW, iOutputResH / 2, iLowerBackdrop);

	/*
	 * Draw the date/current channel box box.
	 */
	cx = YaepgSetup.LeftMargin + 4, cy = 2;
	char *ptr;
	char dateStr[16];
	char condense[10];
	strncpy(condense, *cSource::ToString(source_list[iChanSource]), 10);

	cTextBox *datebox;
	if (iShowAll == 1) {
		datebox = new cTextBox(CHBOX_WIDTH + iChanWidthModifer, TMBOX_HEIGHT, iWhite, iHighlight, fontTimeBox);
		if (currChan == 0) {
			snprintf(dateStr, 16, "%d.%d", startTime.tm_mday, startTime.tm_mon+1);
		} else {
			snprintf(dateStr, 16, "%d", currChan);
		}
	} else {
		if (currChan == 0) {
			datebox = new cTextBox(CHBOX_WIDTH + iChanWidthModifer, TMBOX_HEIGHT, iWhite, iLowerBackdrop, fontTimeBox);
			/* get rid of the S so we can condense the sourcename so it fits better */
			ptr = condense;
			while ((ptr = strchr(ptr, 'S')) != NULL)
				memmove(ptr, ptr+1, strlen(ptr));
			snprintf(dateStr, 16, "%s", condense);
		} else {
			datebox = new cTextBox(CHBOX_WIDTH + iChanWidthModifer, TMBOX_HEIGHT, iWhite, iHighlight, fontTimeBox);
			snprintf(dateStr, 16, "%d", currChan);
		}
	}
	datebox->SetXAlign(TBOX_TXT_CENTER);
	datebox->SetText(1, dateStr);
	datebox->Draw(cx, cy, EPGBitmap);

	/*
	 * Draw the time bar and the timeline.
	 */
	if (startT <= currT) {
		int min;
		min = currTime.tm_min - ((startTime.tm_min / 30) * 30);
		cx = YaepgSetup.LeftMargin + CHBOX_WIDTH + iChanWidthModifer + 7 + (int)(((float)min * 4.86) + 0.5);
		EPGBitmap->DrawRectangle(cx, 20, cx+1, 212, iWhite);
	}
	cx = YaepgSetup.LeftMargin + CHBOX_WIDTH + iChanWidthModifer + 7, cy = 2;
	for (i = 0; i < 4; i++) {
		timebox[i] = new cTimeBox(&startTime, i*30);
		timebox[i]->Draw(cx, cy, EPGBitmap);
		cx += timebox[i]->Width();
	}

	/*
	 * Draw the channel bar
	 */
	cx = YaepgSetup.LeftMargin + 4, cy = 22;
	GetChannelListEnds();
	if ((startChan < first_channel) || (startChan > last_channel)) {
		startChan = first_channel;
	}
	c = Channels.GetByNumber(startChan);
	for (i = 0; i < iChannelCount; i++) {
		if (c != NULL) {
			if (iShowAll != 1) {
				if (c->Source() == source_list[iChanSource]) {
#ifdef FAVORITES_AND_SORT_PATCH
					if (iShowFavorites == 1) {
						if (c->IsFavorite()) {
							chans[i] = new cChanBox(c);
							chans[i]->Draw(cx, cy, EPGBitmap);
							cy += chans[i]->Height() + 2;
						} else {
							i--;
						}				
					} else {
#endif
						chans[i] = new cChanBox(c);
						chans[i]->Draw(cx, cy, EPGBitmap);
						cy += chans[i]->Height() + 2;
#ifdef FAVORITES_AND_SORT_PATCH
					}
#endif
				} else {
					i--;
				}
			} else {
#ifdef FAVORITES_AND_SORT_PATCH
				if (iShowFavorites == 1) {
					if (c->IsFavorite()) {
						chans[i] = new cChanBox(c);
						chans[i]->Draw(cx, cy, EPGBitmap);
						cy += chans[i]->Height() + 2;
					} else {
						i--;
					}				
				} else {
#endif
					chans[i] = new cChanBox(c);
					chans[i]->Draw(cx, cy, EPGBitmap);
					cy += chans[i]->Height() + 2;
#ifdef FAVORITES_AND_SORT_PATCH
				}
#endif
			}
		}
                if (YaepgSetup.ChannelOrder == CHANNEL_ORDER_UP) {
                   while ((c = (cChannel *)c->Prev()) && (c->GroupSep()));
                   if (c == NULL)
                      c = Channels.GetByNumber(last_channel);
                } else {
                   while ((c = (cChannel *)c->Next()) && (c->GroupSep()));
                   if (c == NULL) 
                      c = Channels.GetByNumber(first_channel);
		}
	}

	/*
	 * Draw the event grid
	 */
	cSchedulesLock SchedulesLock;
	const cSchedules* Schedules = cSchedules::Schedules(SchedulesLock);
	cy = 22;
	time_t grTime, t0 = mktime(&timebox[0]->tmTime);
	const cSchedule *currSched;
	const cEvent *currEvent;
	for (i = 0; i < iChannelCount; i++) {
		cx = YaepgSetup.LeftMargin + CHBOX_WIDTH + iChanWidthModifer + 6;
		currSched = Schedules->GetSchedule(chans[i]->GetChanID());

		/*
		 * I assume that there won't be a channel that has more than 8
		 * events in a 2 hour time period.  This seems reasonable, if
		 * there are more than 8 it just won't show them.
		 */
		j = 0;
		grTime = t0;
		eventgrid[i].rowLen = 0;
		while ((j < 8) && (grTime < (mktime(&timebox[3]->tmTime) + 1800))) {
			if (currSched != NULL) {
				currEvent = currSched->GetEventAround(grTime);
				if ((currEvent != NULL) &&
					(currEvent->StartTime() + currEvent->Duration()) <= grTime) {
					currEvent = NULL;
				}
			}
			else {
				currEvent = NULL;
			}

			if (currEvent == NULL) {
				/*
				 * If we didn't find an event at this time try
				 * make a "No Info" evnet.
				 */
				currEvent = new cNoInfoEvent(grTime);
				if (currEvent == NULL) {
					fprintf(stderr, "NULL\n");
				}
			}

#if 0
			fprintf(stderr, "%d %d %s\n",
					currEvent->StartTime(),
					currEvent->Duration(),
					currEvent->Title());
#endif

			eventgrid[i].event[j] = new cEventBox(currEvent, t0);
			eventgrid[i].rowLen++;
			eventgrid[i].event[j]->Draw(
					cx + eventgrid[i].event[j]->XOff(),
					cy, EPGBitmap);
			grTime = currEvent->StartTime() +
				currEvent->Duration();
			j++;
		}
		cy += CHBOX_HEIGHT + iEPGMarginY + 2;
	}

	if ((scrollDir == SCROLL_LEFT) || (cursorX >= eventgrid[cursorY].rowLen)) {
		cursorX = (eventgrid[cursorY].rowLen-1);
	}
	eventgrid[cursorY].event[cursorX]->SetHighLight(true, true);
	eventgrid[cursorY].event[cursorX]->Draw();
	selEvent = eventgrid[cursorY].event[cursorX]->GetEvent();

	selChan = chans[cursorY]->GetChanNum();

	osd->DrawBitmap(0, iOutputResH / 2, *EPGBitmap);

	/*
	 * Draw the info window
	 */
	InfoBitmap->DrawRectangle(0, 0, iOutputResW, iOutputResH / 2 - 1, iUpperBackdrop);

	cx = YaepgSetup.LeftMargin;
	cy = iTop;
	identbox = new cIdentBox(currT);
	identbox->Draw(cx, cy, InfoBitmap);

	cx = YaepgSetup.LeftMargin + iIdboxWidth;
	cy = iTop;
	datebox = new cDateBox(currT);
	datebox->Draw(cx, cy, InfoBitmap);

	cx = iInfboxOffset;
	cy = iTop + 20;
	durbox = new cDurBox(eventgrid[cursorY].event[cursorX]->GetEvent());
	durbox->Draw(cx, cy, InfoBitmap);

	cx = iInfboxOffset;
	cy = iTop + 59;
	infobox = new cInfoBox(eventgrid[cursorY].event[cursorX]->GetEvent());
	infobox->Draw(cx, cy, InfoBitmap);

	cx = iOutputResW + YaepgSetup.LeftMargin - iLeftMarginLimit - iSrcboxWidth - 20;
	cy = (iOutputResH / 2)- 21;
	sourcebox = new cSourceBox(eventgrid[cursorY].event[cursorX]->GetEvent());
	sourcebox->Draw(cx, cy, InfoBitmap);

	/* draw this last so that the tails of 'g,y' etc show up */
	cx = iInfboxOffset;
	cy = iTop + 40;
	titlebox = new cTitleBox(eventgrid[cursorY].event[cursorX]->GetEvent());
	titlebox->Draw(cx, cy, InfoBitmap);

	osd->DrawBitmap(0, 0, *InfoBitmap);

	/*
	 * Draw the message box if needed.
	 */
	if (msgbox != NULL) {
		msgbox->Draw();
		osd->DrawBitmap((iOutputResW - MSGBOX_WIDTH) / 2,
				((iOutputResH / 2) + ((iOutputResH / 2) - MSGBOX_HEIGHT) / 2),
				*(msgbox->GetBmp()));
	}

	/*
	 * Draw the message dialog box if needed.
	 */
	if (msgDlgBox != NULL) {
		msgDlgBox->Draw();
		osd->DrawBitmap((iOutputResW - MSGBOX_WIDTH) / 2,
				((iOutputResH / 2) + ((iOutputResH / 2) - MSGBOX_HEIGHT) / 2),
				*(msgDlgBox->GetBmp()));
	}

	/*
	 * Draw the record dialog box if needed.
	 */
	if (recDlgBox != NULL) {
		recDlgBox->Draw();
		osd->DrawBitmap((iOutputResW - RECBOX_WIDTH) / 2,
				((iOutputResH / 2) + ((iOutputResH / 2) - RECBOX_HEIGHT) / 2),
				*(recDlgBox->GetBmp()));
	}

	/*
	 * Flush everything to the screen.
	 */
	osd->Flush();

	/*
	 * Free everything.  It would be a lot more efficient to just leave
	 * these things allocated and update the info in them.
	 */
	for (int i = 0; i < 4; i++) {
		delete timebox[i];
	}

	for (int i = 0; i < iChannelCount; i++) {
		delete chans[i];
		for (int j = 0; j < eventgrid[i].rowLen; j++) {
			delete eventgrid[i].event[j];
		}
	}

	delete durbox;
	delete titlebox;
	delete infobox;
	delete datebox;
	delete identbox;
	delete sourcebox;
}

eOSState
cYaepg::ProcessKey(eKeys Key)
{
	eOSState state = cOsdObject::ProcessKey(Key);
	if (recDlgBox != NULL && state == osUnknown) {
		switch (Key & ~k_Repeat) {
		case kOk:
			/* Add the timer */
			recDlgBox->AddTimer();
			delete recDlgBox;
			recDlgBox = NULL;
			
			/* Display the "Timer added" message" */
			msgbox = new cMessageBox(tr("Timer added"));
			msgboxStart = cTimeMs::Now();

			Draw();
			break;
		case kBack:
			/* Cancel adding a timer */
			delete recDlgBox;
			recDlgBox = NULL;

			/* Display the "Timer cancelled" message */
			msgbox = new cMessageBox(tr("Timer cancelled"));
			msgboxStart = cTimeMs::Now();

			Draw();
			break;
		default:
			if (recDlgBox->ProcessKey(Key) == true) {
				Draw();
			}
			break;
		}

		return (osContinue);
	}
	if (msgDlgBox != NULL && state == osUnknown) {
		switch (Key & ~k_Repeat) {
		case kOk:
			if (selEvent->HasTimer()) {
			   for (cTimer *timer = Timers.First(); timer; timer = Timers.Next(timer)) {
 				if (timer->Event() == selEvent) {
				   /* Delete the timer */
				   Timers.Del(timer);
				   Timers.Save();
				   delete msgDlgBox;
				   msgDlgBox = NULL;

				   /* Display the "Timer deleted" message */
				   msgbox = new cMessageBox(tr("Timer deleted"));
				   msgboxStart = cTimeMs::Now();

				   Draw();
				   break;
				}
			    }
			}
			break;
		case kBack:
			if (selEvent->HasTimer()) {
			   for (cTimer *timer = Timers.First(); timer; timer = Timers.Next(timer)) {
 				if (timer->Event() == selEvent) {
				   /* Cancel deleting a timer */
				   delete msgDlgBox;
				   msgDlgBox = NULL;

				   /* Display the "Timer deletion cancelled" message */
				   msgbox = new cMessageBox(tr("Timer deletion cancelled"));
				   msgboxStart = cTimeMs::Now();

				   Draw();
				   break;
				}
			    }
			}
			break;
		default:
			break;
		}

		return (osContinue);
	}
	if (state == osUnknown) {
		switch (Key & ~k_Repeat) {
		case kBack:
#ifdef XINE_UI_ZOOM_PATCH
			iZoomToPrevious = 1;
#endif
			state = osEnd;
			break;

		case kLeft:
			MoveCursor(CURSOR_LEFT);
			Draw();
			state = osContinue;
			break;

		case kRight:
			MoveCursor(CURSOR_RIGHT);
			Draw();
			state = osContinue;
			break;

		case kUp:
			MoveCursor(CURSOR_UP);
			Draw();

			if (YaepgSetup.ChannelChange == CHANNEL_CHANGE_AUTO_INEPG) {
				SwitchToCurrentChannel();
                        }
			state = osContinue;
			break;

		case kDown:
			MoveCursor(CURSOR_DOWN);
			Draw(); 

			if (YaepgSetup.ChannelChange == CHANNEL_CHANGE_AUTO_INEPG) {
				SwitchToCurrentChannel();
			}
			state = osContinue;
			break;

		case kOk:
			/*
			 * If OK is pressed on an event that hasn't started yet,
			 * then we want to bring up the "record" dialog box.
			 */
			fprintf(stdout,"OK button\n");
			if (selEvent->StartTime() <= currT) {
			fprintf(stdout,"OK button 2\n");
				SwitchToCurrentChannel();
				if (YaepgSetup.ChannelChange == CHANNEL_CHANGE_MANUAL_INEPG)
				   state = osContinue;
				else
#ifdef XINE_UI_ZOOM_PATCH
				   if (YaepgSetup.ZoomReset == true)
				      iZoomToPrevious = 0;
#endif
				   state = osEnd;
			}
			else {
				if (selEvent->HasTimer()) {
				   for (cTimer *timer = Timers.First(); timer; timer = Timers.Next(timer)) {
 			     		if (timer->Event() == selEvent) {
				   		msgDlgBox = new cMsgDlgBox(tr("Delete this timer?"));
						break;
					}
				    }
				}
				else {
				   recDlgBox = new cRecDlgBox(selEvent);
				}
				Draw();
			}
			break;

		case kRed:
		case kGreen:
		case kYellow:
		case kBlue:
			if (((Key == kRed) && (YaepgSetup.FnRed == 0)) || ((Key == kGreen) && (YaepgSetup.FnGreen == 0)) ||
			   ((Key == kYellow) && (YaepgSetup.FnYellow == 0)) || ((Key == kBlue) && (YaepgSetup.FnBlue == 0))) {
				/* Page Up */
				ChangeChan((YaepgSetup.ChannelOrder == CHANNEL_ORDER_UP ? 1 : -1) * iChannelCount * 1);
			}
			if (((Key == kRed) && (YaepgSetup.FnRed == 1)) || ((Key == kGreen) && (YaepgSetup.FnGreen == 1)) ||
			   ((Key == kYellow) && (YaepgSetup.FnYellow == 1)) || ((Key == kBlue) && (YaepgSetup.FnBlue == 1))) {
				/* Page Down */
				ChangeChan((YaepgSetup.ChannelOrder == CHANNEL_ORDER_UP ? 1 : -1) * iChannelCount * -1);
			}
			if (((Key == kRed) && (YaepgSetup.FnRed == 2)) || ((Key == kGreen) && (YaepgSetup.FnGreen == 2)) ||
			   ((Key == kYellow) && (YaepgSetup.FnYellow == 2)) || ((Key == kBlue) && (YaepgSetup.FnBlue == 2))) {
				/* Switch Channel */
				SwitchToCurrentChannel();
			}
			if (((Key == kRed) && (YaepgSetup.FnRed == 3)) || ((Key == kGreen) && (YaepgSetup.FnGreen == 3)) ||
			   ((Key == kYellow) && (YaepgSetup.FnYellow == 3)) || ((Key == kBlue) && (YaepgSetup.FnBlue == 3))) {
				/* Switch Source */
				if (iShowAll == 1) {	
					iShowAll = 0;
				} else {
					if (++iChanSource == iLastSource) {
            					iChanSource = 0;
						iShowAll = 1;
					}
				}
			}
			Draw();
			state = osContinue;
			break;

		case k0 ... k9:
#ifdef FAVORITES_AND_SORT_PATCH
			if ((!currChan) && (Key == k0)) {
				/* Toggle Sort mode */
				if (iShowFavorites == 1) {	
					iShowFavorites = 0;
					msgbox = new cMessageBox(tr("Sort by number"));
				} else {
					if (iHasFavorites == 1) {
						iShowFavorites = 1;
						msgbox = new cMessageBox(tr("Sort by favorite"));
					}
				}
				msgboxStart = cTimeMs::Now();
				Draw();
				state = osContinue;
				break;
			}
#endif
			/* Direct Channel Input */
			if ((currChan) || (Key != k0)) {
				cChannel *chan = Channels.GetByNumber(currChan);
				cChannel *ch = chan;

				currChan = (currChan * 10) + (Key - k0);
				int n = chan ? currChan * 10 : 0;
				while (ch && (ch = Channels.Next(ch)) != NULL) {
					if (n <= ch->Number() && ch->Number() <= (n + 9)) {
						n = 0;
						break;
					}
					if (ch->Number() > n) {
						n *= 10;
					}
				}
				if (n > 0) {
					if (Channels.GetByNumber(currChan)) {
						startChan = currChan;
						Draw();
					}
					state = osContinue;
					break;
				}

				lastInput = cTimeMs::Now();
				Draw();
			}
			break;

		default:
			break;
		}
	}

	/*
	 * Check for timer related stuff on ANY key press.  Note, this code used
	 * to be part of kNone key processing but if users keep pressing keys we
	 * won't get a kNnone for a while.  This makes the timer related
	 * operations more responsive.
	 */
	time_t nowT;
	struct tm nowTm;
	bool needDraw = false;
	uiChannelEntryTimeout = YaepgSetup.ChanEntTimeout;

	(void) time(&nowT);
	(void) localtime_r(&nowT, &nowTm);

	/* Update the current time line ? */
	if (nowTm.tm_min != startTime.tm_min) {
		UpdateTime();
		needDraw = true;
	}

	/* Cancel direct channel input ? */
	if (currChan && (cTimeMs::Now() - lastInput) > uiChannelEntryTimeout) {
		/* fprintf(stderr, "Timeout! %d\n", currChan); */
		/* if channel doesn't exist find closest channel within 100 (down or up) */
		for (int i = 0; i < 100; i++) {
			if (Channels.GetByNumber(currChan-i)) {
				cursorY = 0;
				startChan = currChan-i;
				needDraw = true;
				break;
			}
			if (Channels.GetByNumber(currChan+i)) {
				cursorY = 0;
				startChan = currChan+i;
				needDraw = true;
				break;
			}
		}
		currChan = 0;
	}

	/* Did the message box expire ? */
	if (msgbox != NULL && (cTimeMs::Now() - msgboxStart) > uiChannelEntryTimeout) {
		delete msgbox;
		msgbox = NULL;
		needDraw = true;
	}

	/* Did something happen that requires a redraw ? */
	if (needDraw == true) {
		Draw();
	}

	return state;
}

void
cYaepg::ChangeChan(int change)
{
	cChannel *c;

	c = Channels.GetByNumber(startChan);
	GetChannelListEnds();
	if (change > 0) {
	        if (startChan > last_channel) {
			startChan = first_channel;
			return;
		}
		for (int i = 0; i < change; i++) {
			while ((c = (cChannel *)c->Next()) && (c->GroupSep())) {
				;
			}
			if (c == NULL) {
				c = Channels.GetByNumber(first_channel);
			}
			startChan = c->Number();
			if (iShowAll != 1) {
				if (Channels.GetByNumber(startChan)->Source() != source_list[iChanSource]) {
					i--;
#ifdef FAVORITES_AND_SORT_PATCH
				} else {
					if (iShowFavorites == 1) {	
						if (!c->IsFavorite())
							i--;
					}
				}
			} else {
				if (iShowFavorites == 1) {	
					if (!c->IsFavorite())
						i--;
#endif
				}			
			}
		}
	}
	else if (change < 0) {
	        if (startChan < first_channel) {
			startChan = last_channel;
			return;
		}
		for (int i = 0; i > change; i--) {
			while ((c = (cChannel *)c->Prev()) && (c->GroupSep())) {
				;
			}
			if (c == NULL) {
				c = Channels.GetByNumber(last_channel);
			}
			startChan = c->Number();
			if (iShowAll != 1) {
				if (Channels.GetByNumber(startChan)->Source() != source_list[iChanSource]) {
					i++;
#ifdef FAVORITES_AND_SORT_PATCH
				} else {
					if (iShowFavorites == 1) {	
						if (!c->IsFavorite())
							i++;
					}
				}
			} else {
				if (iShowFavorites == 1) {	
					if (!c->IsFavorite())
						i++;
#endif
				}
			}
		}
	}
}

int
cYaepg::ChangeTime(int shift)
{
	/*
	 * Don't let the guide go back in time.
	 */
	if ((startT + shift) < firstT) {
		return -1;
	}

	startT += shift;
	(void) localtime_r(&startT, &startTime);

	return 0;
}

void
cYaepg::MoveCursor(int dir)
{
	prevEventId = 0;
	scrollDir = 0;

	switch (dir) {
	case CURSOR_UP:
		if (cursorY == 0) {
			ChangeChan(1 * (YaepgSetup.ChannelOrder == CHANNEL_ORDER_UP ? 1 : -1));
		}
		else {
			cursorY--;
		}
		break;

	case CURSOR_DOWN:
		if (cursorY == iChannelCount - 1) {
			ChangeChan(-1 * (YaepgSetup.ChannelOrder == CHANNEL_ORDER_UP ? 1 : -1));
		}
		else {
			cursorY++;
		}
		break;

	case CURSOR_LEFT:
		if (cursorX == 0) {
			if (ChangeTime(-5400) == 0) {
				scrollDir = SCROLL_LEFT;
			}
		}
		else {
			cursorX--;
		}
		break;

	case CURSOR_RIGHT:
		if (cursorX == (eventgrid[cursorY].rowLen - 1)) {
			(void) ChangeTime(5400);
			scrollDir = SCROLL_RIGHT;
			cursorX = 0;
		}
		else {
			cursorX++;
		}
		break;

	default:
		break;
	}

	if (cursorX >= eventgrid[cursorY].rowLen) {
		cursorX = eventgrid[cursorY].rowLen - 1;
	}
}

void
cYaepg::UpdateTime(void)
{
	(void) time(&currT);
	(void) localtime_r(&currT, &currTime);

	if ((currTime.tm_min == 0) || (currTime.tm_min == 30)) {
		firstT = AdjustTime(currT);
		(void) localtime_r(&firstT, &firstTime);
		/*
		 * Do we need to update startTime ?
		 */
		if (startT < firstT) {
			startT = firstT;
			(void) localtime_r(&startT, &startTime);
		}
	}
}

void
cYaepg::LogMsg(eYlog_t logType, const char *fmt, ...)
{
	va_list ap;

	if (log) {
		switch (logType) {
		case YLOG_ERR:
			fprintf(log, "ERROR: ");
			break;
		case YLOG_INF:
			fprintf(log, "INFO: ");
			break;
		default:
			return;
		}
		va_start(ap, fmt);
		vfprintf(log, fmt, ap);
		va_end(ap);
		fflush(log);
	}
	else {
		fprintf(stderr, "LOG = NULL!!!\n");
	}
}

/*
 * Round the time down to a 30 minute boundary
 */
time_t
cYaepg::AdjustTime(time_t t)
{
	struct tm tm;

	(void) localtime_r(&t, &tm);
	tm.tm_sec = 0;
	if (tm.tm_min < 30) {
		tm.tm_min = 0;
	}
	else {
		tm.tm_min = 30;
	}

	return mktime(&tm);
}

class cPluginYaepg : public cPlugin {
private:
	/* Add any member variables or functions you may need here. */
	cYaepg			*yaepg;
	
public:
	cPluginYaepg(void);
	virtual ~cPluginYaepg();
	virtual const char *Version(void) { return VERSION; }
	virtual const char *Description(void) { return DESCRIPTION; }
	virtual const char *CommandLineHelp(void);
	virtual bool ProcessArgs(int argc, char *argv[]);
	virtual bool Initialize(void);
	virtual bool Start(void);
	virtual void Housekeeping(void);
	virtual const char *MainMenuEntry(void) { return YaepgSetup.HideMenuEntry ? NULL : MAINMENUENTRY; }
	virtual cOsdObject *MainMenuAction(void);
	virtual cMenuSetupPage *SetupMenu(void);
	virtual bool SetupParse(const char *Name, const char *Value);
};

cPluginYaepg::cPluginYaepg(void)
{
	/*
	 * Initialize any member variables here.
	 * DON'T DO ANYTHING ELSE THAT MAY HAVE SIDE EFFECTS, REQUIRE GLOBAL
	 * VDR OBJECTS TO EXIST OR PRODUCE ANY OUTPUT!
	 */
}

cPluginYaepg::~cPluginYaepg()
{
	/* Clean up after yourself! */
}

const char *cPluginYaepg::CommandLineHelp(void)
{
	/* Return a string that describes all known command line options. */
	return NULL;
}

bool cPluginYaepg::ProcessArgs(int argc, char *argv[])
{
	/* Implement command line argument processing here if applicable. */
	return true;
}

bool cPluginYaepg::Initialize(void)
{
	/* Initialize any background activities the plugin shall perform. */
	return true;
}

void cPluginYaepg::Housekeeping(void)
{
	/* Perform any cleanup or other regular tasks here if needed. */
}

cOsdObject *cPluginYaepg::MainMenuAction(void)
{
	/* Perform the action when selected from the main VDR menu. */
	yaepg = new cYaepg;

	return yaepg;
}

cMenuSetupPage *cPluginYaepg::SetupMenu(void)
{
  /* Return the setup menu. */
  return new cYaepgSetupMenu;
}

bool cPluginYaepg::SetupParse(const char *Name, const char *Value)
{
  /* Parse the setup parameters. */
  return YaepgSetup.SetupParse(Name, Value);
}


VDRPLUGINCREATOR(cPluginYaepg); /* Don't touch this! */
