/*
 * yaepgSetupMenu.c: A plugin for the Video Disk Recorder
 *
 * See the README file for copyright information and how to reach the author.
 *
 */

#include "yaepg.h"
#include "yaepgSetupMenu.h"

cYaepgSetup YaepgSetup;

cYaepgSetup::cYaepgSetup(void)
{
  /* set initial default values to be used if none are present in setup.conf */
  YaepgSetup.TvFormat		= TV_NTSC;
  YaepgSetup.Theme		= DEFAULT_THEME;
  YaepgSetup.SkinStyle		= SKIN_TRANSPARENT;
  YaepgSetup.ScreenMode		= MODE_FULLSCREEN;
  YaepgSetup.LeftMargin		= 0; /* set to zero to correspond with fullscreen mode */
#ifdef SWITCH_AND_RECORD_PATCH
  YaepgSetup.DefaultTimer	= TIMER_SYSTEM_DEFAULT;
#else
  YaepgSetup.DefaultTimer	= TIMER_RECORD;
#endif
  YaepgSetup.DefaultUseEpisode	= USE_EPISODE_SMART;
  YaepgSetup.ChanEntTimeout	= DEFAULT_CHANNEL_ENTRY_TIMEOUT;
  YaepgSetup.ChannelChange	= CHANNEL_CHANGE_MANUAL;
  YaepgSetup.ChannelOrder	= CHANNEL_ORDER_DOWN;
  YaepgSetup.TimeFormat		= TIME_24HR;
#ifdef XINE_UI_ZOOM_PATCH
  YaepgSetup.ZoomReset		= true;
  YaepgSetup.ToggleInterlace	= false;
#endif
  YaepgSetup.HideMenuEntry	= false;
  YaepgSetup.FnRed		= FN_SWITCH_CHANNEL;
  YaepgSetup.FnGreen		= FN_PAGE_UP;
  YaepgSetup.FnYellow		= FN_PAGE_DOWN;
  YaepgSetup.FnBlue		= FN_SWITCH_SOURCE;
}


bool cYaepgSetup::SetupParse(const char *Name, const char *Value)
{
  if      (!strcasecmp(Name, SETUP_PARAM_TVFORMAT))		TvFormat		= atoi(Value);
  else if (!strcasecmp(Name, SETUP_PARAM_THEME))		Theme			= atoi(Value);
  else if (!strcasecmp(Name, SETUP_PARAM_SKINSTYLE))		SkinStyle		= atoi(Value);
  else if (!strcasecmp(Name, SETUP_PARAM_SCREENMODE))		ScreenMode		= atoi(Value);
  else if (!strcasecmp(Name, SETUP_PARAM_LMARGIN))		LeftMargin		= atoi(Value);
#ifdef SWITCH_AND_RECORD_PATCH
  else if (!strcasecmp(Name, SETUP_PARAM_DEFAULTTIMER))		DefaultTimer		= atoi(Value);
#endif
  else if (!strcasecmp(Name, SETUP_PARAM_DEFAULTUSEEPISODE))	DefaultUseEpisode	= atoi(Value);
  else if (!strcasecmp(Name, SETUP_PARAM_CHENTRY))		ChanEntTimeout		= atoi(Value);
  else if (!strcasecmp(Name, SETUP_PARAM_CHCHANGE))		ChannelChange		= atoi(Value);
  else if (!strcasecmp(Name, SETUP_PARAM_CHORDER))		ChannelOrder		= atoi(Value);
  else if (!strcasecmp(Name, SETUP_PARAM_TIMEFORMAT))		TimeFormat		= atoi(Value);
#ifdef XINE_UI_ZOOM_PATCH
  else if (!strcasecmp(Name, SETUP_PARAM_ZOOMRESET))		ZoomReset		= atoi(Value);
  else if (!strcasecmp(Name, SETUP_PARAM_INTERLACE))		ToggleInterlace		= atoi(Value);
#endif
  else if (!strcasecmp(Name, SETUP_PARAM_HIDEMENU))		HideMenuEntry		= atoi(Value);
  else if (!strcasecmp(Name, SETUP_PARAM_RED))			FnRed			= atoi(Value);
  else if (!strcasecmp(Name, SETUP_PARAM_GREEN))		FnGreen			= atoi(Value);
  else if (!strcasecmp(Name, SETUP_PARAM_YELLOW))		FnYellow		= atoi(Value);
  else if (!strcasecmp(Name, SETUP_PARAM_BLUE))			FnBlue			= atoi(Value);

  else return false;

  return true;
}

cYaepgSetupMenu::cYaepgSetupMenu(void)
{

  TV_FORMATS[0]		= tr("720x480");
  TV_FORMATS[1]		= tr("720x576");

  THEME_NAMES[0]	= tr("default");
  THEME_NAMES[1]	= tr("contrast");
  THEME_NAMES[2]	= tr("theater");

  SKIN_STYLES[0]	= tr("transparent");
  SKIN_STYLES[1]	= tr("solid");
  SKIN_STYLES[2]	= tr("windowed");
  SKIN_STYLES[3]	= tr("windowed large");

  SCREEN_MODES[0]	= tr("fullscreen");
  SCREEN_MODES[1]	= tr("condensed");
  SCREEN_MODES[2]	= tr("cropped");

#ifdef SWITCH_AND_RECORD_PATCH
  TIMER_METHODS[0]	= tr("switch");
  TIMER_METHODS[1]	= tr("switch & record");
  TIMER_METHODS[2]	= tr("record");
  TIMER_METHODS[3]	= tr("system default");
#endif

  USE_EPISODE_MODES[0]	= tr("never");
  USE_EPISODE_MODES[1]	= tr("smart");
  USE_EPISODE_MODES[2]	= tr("always");

  CH_CHANGE_MODES[0]	= tr("manual");
  CH_CHANGE_MODES[1]	= tr("manual in EPG");
  CH_CHANGE_MODES[2]	= tr("auto in EPG");

  CH_ORDER_FORMATS[0]	= tr("up");
  CH_ORDER_FORMATS[1]	= tr("down");

  TIME_FORMATS[0]	= tr("24");
  TIME_FORMATS[1]	= tr("12");

  RED[0]		= tr("page up");
  RED[1]		= tr("page down");
  RED[2]		= tr("switch channel");
  RED[3]		= tr("switch source");

  GREEN[0]		= tr("page up");
  GREEN[1]		= tr("page down");
  GREEN[2]		= tr("switch channel");
  GREEN[3]		= tr("switch source");

  YELLOW[0]		= tr("page up");
  YELLOW[1]		= tr("page down");
  YELLOW[2]		= tr("switch channel");
  YELLOW[3]		= tr("switch source");

  BLUE[0]		= tr("page up");
  BLUE[1]		= tr("page down");
  BLUE[2]		= tr("switch channel");
  BLUE[3]		= tr("switch source");

}

eOSState
cYaepgSetupMenu::ProcessKey(eKeys Key)
{
  int current=Current();
  eOSState state = cOsdMenu::ProcessKey(Key);

  switch (Key & ~k_Repeat) {
    case kUp:
    case kDown:
    case k0 ... k9:
      /* ignore up, down and number keys so that we can navigate the menu and enter values */
      break;
    case kOk:
      Store();
      state = osBack;
      break;
    default:
      Clear();
      Add(new cMenuEditStraItem (tr("OSD resolution"),		   &YaepgSetup.TvFormat,	2,   TV_FORMATS));
      Add(new cMenuEditStraItem (tr("Theme"),			   &YaepgSetup.Theme,		3,   THEME_NAMES));
      Add(new cMenuEditStraItem (tr("Skin style"),		   &YaepgSetup.SkinStyle,	4,   SKIN_STYLES));
      Add(new cMenuEditStraItem (tr("Screen mode"),		   &YaepgSetup.ScreenMode,	3,   SCREEN_MODES));
      if (YaepgSetup.ScreenMode == MODE_FULLSCREEN) {
        YaepgSetup.LeftMargin = 0;
      }
      else {
           if (YaepgSetup.LeftMargin == 0) {
             YaepgSetup.LeftMargin = DEFAULT_LEFT_MARGIN;
           }
      Add(new cMenuEditIntItem  (tr("Left margin"),		   &YaepgSetup.LeftMargin,	0,   MAX_LEFT_MARGIN));
      }
#ifdef SWITCH_AND_RECORD_PATCH
      Add(new cMenuEditStraItem (tr("Default timer"),	   	   &YaepgSetup.DefaultTimer,	4,   TIMER_METHODS));
#endif
      Add(new cMenuEditStraItem (tr("Use episode in timer logic"), &YaepgSetup.DefaultUseEpisode, 3, USE_EPISODE_MODES));
      Add(new cMenuEditIntItem  (tr("Channel entry timout (ms)"),  &YaepgSetup.ChanEntTimeout,	500, MAX_ENTRY_TIMEOUT));
      Add(new cMenuEditStraItem (tr("Channel change"),		   &YaepgSetup.ChannelChange,	3,   CH_CHANGE_MODES));
      Add(new cMenuEditStraItem (tr("Channel order"),		   &YaepgSetup.ChannelOrder,	2,   CH_ORDER_FORMATS));
      Add(new cMenuEditStraItem (tr("Time format"),		   &YaepgSetup.TimeFormat,	2,   TIME_FORMATS));
#ifdef XINE_UI_ZOOM_PATCH
      Add(new cMenuEditBoolItem (tr("Zoom reset on channel change"), &YaepgSetup.ZoomReset));
      Add(new cMenuEditBoolItem (tr("Xine interlace toggle (post)"), &YaepgSetup.ToggleInterlace));
#endif
      Add(new cMenuEditBoolItem (tr("Hide mainmenu entry"),	   &YaepgSetup.HideMenuEntry));
      Add(new cMenuEditStraItem (tr("Red"),			   &YaepgSetup.FnRed,		4,   RED));
      Add(new cMenuEditStraItem (tr("Green"),			   &YaepgSetup.FnGreen,		4,   GREEN));
      Add(new cMenuEditStraItem (tr("Yellow"),			   &YaepgSetup.FnYellow,	4,   YELLOW));
      Add(new cMenuEditStraItem (tr("Blue"),			   &YaepgSetup.FnBlue,		4,   BLUE));
      SetCurrent(Get(current));
      Display();
      break;  
  }
  
  return state;
}

void cYaepgSetupMenu::Store(void)
{
  SetupStore(SETUP_PARAM_TVFORMAT,		YaepgSetup.TvFormat );
  SetupStore(SETUP_PARAM_THEME,			YaepgSetup.Theme );
  SetupStore(SETUP_PARAM_SKINSTYLE,		YaepgSetup.SkinStyle );
  SetupStore(SETUP_PARAM_SCREENMODE,		YaepgSetup.ScreenMode );
  SetupStore(SETUP_PARAM_LMARGIN,		YaepgSetup.LeftMargin );
#ifdef SWITCH_AND_RECORD_PATCH
  SetupStore(SETUP_PARAM_DEFAULTTIMER,		YaepgSetup.DefaultTimer );
#endif
  SetupStore(SETUP_PARAM_DEFAULTUSEEPISODE,	YaepgSetup.DefaultUseEpisode );
  SetupStore(SETUP_PARAM_CHENTRY,		YaepgSetup.ChanEntTimeout );
  SetupStore(SETUP_PARAM_CHCHANGE,		YaepgSetup.ChannelChange );
  SetupStore(SETUP_PARAM_CHORDER,		YaepgSetup.ChannelOrder );
  SetupStore(SETUP_PARAM_TIMEFORMAT,		YaepgSetup.TimeFormat );
#ifdef XINE_UI_ZOOM_PATCH
  SetupStore(SETUP_PARAM_ZOOMRESET,		YaepgSetup.ZoomReset );
  SetupStore(SETUP_PARAM_INTERLACE,		YaepgSetup.ToggleInterlace );
#endif
  SetupStore(SETUP_PARAM_HIDEMENU,		YaepgSetup.HideMenuEntry );
  SetupStore(SETUP_PARAM_RED,			YaepgSetup.FnRed );
  SetupStore(SETUP_PARAM_GREEN,			YaepgSetup.FnGreen );
  SetupStore(SETUP_PARAM_YELLOW,		YaepgSetup.FnYellow );
  SetupStore(SETUP_PARAM_BLUE,			YaepgSetup.FnBlue );
}
